# Fields used in LG data structures

# =========================
# ID fields
# =========================
LG_F_SUPPLY_CHAIN_LEVEL_ID = "supplyChainLevelID"  # string, int for defaults, GUID for custom.
LG_F_WAREHOUSE_ID = "warehouseID"
LG_F_EXIST_WAREHOUSE_TYPE_ID = "existWarehouseTypeID"
LG_F_NEW_WAREHOUSE_TYPE_ID = "newWarehouseTypeID"
LG_F_VEHICLE_TYPE_ID = "vehicleTypeID"
LG_F_WORKER_TYPE_ID = "workerTypeID"
LG_F_COLD_CHAIN_ID = "coldChainID"
LG_F_THIRD_PARTY_LOG_CONTRACT_ID = "thirdPartyLogContractID"

# =========================
# Common fields
# =========================
LG_F_NAME = "name"  # string
LG_F_STR_CONST = "stringConstant"  # string;  Defaults only.
LG_F_CUSTOM = "custom"  # bool
# Baseline
LG_F_UNIT_COST = "unitCost"  # int
LG_F_WORKING_LIFE_YEARS = "workingLifeInYears"
LG_F_AVG_AMOUNT_BY_LEVEL_AGGR = "avgAmountByLevelAggr"
LG_F_AVG_AMOUNT_BY_LEVEL_DISAG = "avgAmountByLevelDisag"
LG_F_AMOUNT_BY_WAREHOUSE_DISAG = "amountByWarehouseDisag"
LG_F_ANNUAL_WATER_COST = "annualWaterCost"  # int; If entering component operating costs
LG_F_ANNUAL_ELECTRICAL_COST = "annualElectricalCost"  # int; If entering component operating costs
LG_F_OTHER_OPERATING_COSTS = "otherOperatingCosts"  # int; If entering component operating costs
LG_F_TOTAL_OPERATING_COSTS = "totalOperatingCosts"  # int; If entering costs as a lump sum
# Target setting
LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR = "targetWarehouseStdByLevelAggr"  # dict: supplyChainLevelID
LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG = (
    "targetWarehouseStdByLevelDisag"  # dict by warehouseLevelID by supplyChainLevelID
)
LG_F_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG = "targetWarehouseStdByWarehouseDisag"  # dict by warehouseLevelID
LG_F_TARG_EXISTING_PLANS_BY_LEVEL_AGGR = "targetExistingPlansByLevelAggr"  # numNeededPerLevelAggr
LG_F_TARG_EXISTING_PLANS_BY_LEVEL_DISAG = "targetExistingPlansByLevelDisag"  # numNeededPerLevelDisag
LG_F_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG = "targetExistingPlansByWarehouseDisag"  # numNeededPerWarehouseDisag
LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR = "targetNumAvailableByLevelAggr"  # numAvailablePerLevelAggr
LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG = "targetNumAvailableByLevelDisag"  # numAvailablePerLevelDisag
LG_F_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG = "targetNumAvailableByWarehouseDisag"  # numAvailablePerWarehouseDisag
LG_F_NUM_BUILT_BY_YEAR = "numBuiltByYear"  # 1D list of int
LG_F_LAND_COST = "landCost"  # int
LG_F_TOTAL_CONSTRUCT_COSTS = "totalConstructionCosts"  # int
LG_F_COST_CONSTRUCT_SQ_M = "costOfConstructionPerSqM"  # int
LG_F_SIZE_SQ_M = "sizeInSqM"  # int
LG_F_REHAB_COST = "rehabCost"  # int
LG_F_NUM_REHAB_BY_YEAR = "numRehabByYear"  # 1D list of int
# Volume analytics
LG_F_NUM_RESUPPLY_VISITS_BY_YEAR = "numResupplyVisitsByYear"
LG_F_AVG_PERC_POP_SERVED = "avgPercPopServed"
LG_F_CAPACITY = "capacity"
LG_F_NUM_MONTHS_SAFETY_STOCK = "numMonthsSafetyStock"
LG_F_DISTANCE_TO_SUPPLYING_WAREHOUSE = "distanceToSupplyingWarehouse"

# =========================
# Supply chain level records
# =========================
LG_SUPPLY_CHAIN_LEVEL_ID = LG_F_SUPPLY_CHAIN_LEVEL_ID
LG_SUPPLY_CHAIN_LEVEL_NAME = LG_F_NAME
LG_SUPPLY_CHAIN_LEVEL_STR_CONST = LG_F_STR_CONST
LG_SUPPLY_CHAIN_LEVEL_CUSTOM = LG_F_CUSTOM
LG_SUPPLY_CHAIN_LEVEL_AMOUNT = "amount"  # int
# Baseline
LG_SUPPLY_CHAIN_LEVEL_ANNUAL_WATER_COST = LG_F_ANNUAL_WATER_COST
LG_SUPPLY_CHAIN_LEVEL_ANNUAL_ELECTRICAL_COST = LG_F_ANNUAL_ELECTRICAL_COST
LG_SUPPLY_CHAIN_LEVEL_OTHER_OPERATING_COSTS = LG_F_OTHER_OPERATING_COSTS
LG_SUPPLY_CHAIN_LEVEL_TOTAL_OPERATING_COSTS = LG_F_TOTAL_OPERATING_COSTS
# Target setting
LG_SUPPLY_CHAIN_LEVEL_LAND_COST = LG_F_LAND_COST
LG_SUPPLY_CHAIN_LEVEL_TOTAL_CONSTRUCT_COSTS = LG_F_TOTAL_CONSTRUCT_COSTS
LG_SUPPLY_CHAIN_LEVEL_COST_CONSTRUCT_SQ_M = LG_F_COST_CONSTRUCT_SQ_M
LG_SUPPLY_CHAIN_LEVEL_SIZE_SQ_M = LG_F_SIZE_SQ_M
LG_SUPPLY_CHAIN_LEVEL_NUM_BUILT_BY_YEAR = LG_F_NUM_BUILT_BY_YEAR
LG_SUPPLY_CHAIN_LEVEL_REHAB_COST = LG_F_REHAB_COST
LG_SUPPLY_CHAIN_LEVEL_NUM_REHAB_BY_YEAR = LG_F_NUM_REHAB_BY_YEAR
# Volume analytics
LG_SUPPLY_CHAIN_LEVEL_NUM_RESUPPLY_VISITS_BY_YEAR = LG_F_NUM_RESUPPLY_VISITS_BY_YEAR
LG_SUPPLY_CHAIN_LEVEL_CAPACITY = LG_F_CAPACITY
LG_SUPPLY_CHAIN_LEVEL_NUM_MONTHS_SAFETY_STOCK = LG_F_NUM_MONTHS_SAFETY_STOCK
LG_SUPPLY_CHAIN_LEVEL_DISTANCE_TO_SUPPLYING_WAREHOUSE = LG_F_DISTANCE_TO_SUPPLYING_WAREHOUSE


# =========================
# Warehouse records
# =========================
LG_WAREHOUSE_ID = LG_F_WAREHOUSE_ID
LG_WAREHOUSE_NAME = LG_F_NAME
LG_WAREHOUSE_PARENT_ID = "parentID"  # string
LG_WAREHOUSE_SUPPLY_CHAIN_LEVEL_ID = LG_F_SUPPLY_CHAIN_LEVEL_ID
LG_WAREHOUSE_CUSTOM = LG_F_CUSTOM
LG_WAREHOUSE_TYPE = "warehouseType"  # int
# Baseline
LG_WAREHOUSE_LEVEL_AMOUNTS = "amountsByLevelDisag"
LG_WAREHOUSE_AVG_OPERATING_COSTS_BY_LEVEL = "averageOperatingCostsByLevel"
LG_WAREHOUSE_AVG_CONSTRUCTION_COSTS_BY_LEVEL = "averageConstructionCostsByLevel"
LG_WAREHOUSE_ANNUAL_WATER_COST = LG_F_ANNUAL_WATER_COST
LG_WAREHOUSE_ANNUAL_ELECTRICAL_COST = LG_F_ANNUAL_ELECTRICAL_COST
LG_WAREHOUSE_OTHER_OPERATING_COSTS = LG_F_OTHER_OPERATING_COSTS
LG_WAREHOUSE_TOTAL_OPERATING_COSTS = LG_F_TOTAL_OPERATING_COSTS
# Target setting
LG_WAREHOUSE_LAND_COST = LG_F_LAND_COST
LG_WAREHOUSE_TOTAL_CONSTRUCT_COSTS = LG_F_TOTAL_CONSTRUCT_COSTS
LG_WAREHOUSE_COST_CONSTRUCT_SQ_M = LG_F_COST_CONSTRUCT_SQ_M
LG_WAREHOUSE_SIZE_SQ_M = LG_F_SIZE_SQ_M
LG_WAREHOUSE_NUM_BUILT_BY_YEAR = LG_F_NUM_BUILT_BY_YEAR
LG_WAREHOUSE_NUM_BUILT_BY_YEAR_BY_LEVEL = "numBuiltByYearByLevel"
LG_WAREHOUSE_REHAB_COST = LG_F_REHAB_COST
LG_WAREHOUSE_REHAB_COSTS_BY_LEVEL = "rehabCostsByLevel"
LG_WAREHOUSE_NUM_REHAB_BY_YEAR = LG_F_NUM_REHAB_BY_YEAR
LG_WAREHOUSE_NUM_REHAB_BY_YEAR_BY_LEVEL = "numRehabByYearByLevel"
LG_WAREHOUSE_NUM_MOVED = "numMoved"
LG_WAREHOUSE_NUM_MOVED_BY_LEVEL = "numMovedByLevel"
LG_WAREHOUSE_TARGET_TRANSFER_SOURCE = "targetTransferSource"
LG_WAREHOUSE_TARGET_TRANSFER_SOURCE_BY_LEVEL = "targetTransferSourceByLevel"
LG_WAREHOUSE_YEAR_MOVED = "yearMoved"
LG_WAREHOUSE_YEAR_MOVED_BY_LEVEL = "yearMovedByLevel"
# Volume analytics
LG_WAREHOUSE_NUM_RESUPPLY_VISITS_BY_YEAR = LG_F_NUM_RESUPPLY_VISITS_BY_YEAR
LG_WAREHOUSE_NUM_RESUPPLY_VISITS_BY_YEAR_BY_LEVEL = "numResupplyVisitsByYearByLevel"
LG_WAREHOUSE_AVG_PERC_POP_SERVED = LG_F_AVG_PERC_POP_SERVED
LG_WAREHOUSE_CAPACITY = LG_F_CAPACITY
LG_WAREHOUSE_CAPACITY_BY_LEVEL = "capacityByLevel"
LG_WAREHOUSE_NUM_MONTHS_SAFETY_STOCK = LG_F_NUM_MONTHS_SAFETY_STOCK
LG_WAREHOUSE_NUM_MONTHS_SAFETY_STOCK_BY_LEVEL = "numMonthsSafetyStockByLevel"
LG_WAREHOUSE_DISTANCE_TO_SUPPLYING_WAREHOUSE = LG_F_DISTANCE_TO_SUPPLYING_WAREHOUSE
LG_WAREHOUSE_DISTANCE_TO_SUPPLYING_WAREHOUSE_BY_LEVEL = "distanceToSupplyingWarehouseByLevel"


# =========================
# Vehicle records
# =========================
LG_VEHICLE_TYPE_ID = LG_F_VEHICLE_TYPE_ID
LG_VEHICLE_TYPE_NAME = LG_F_NAME
LG_VEHICLE_TYPE_STR_CONST = LG_F_STR_CONST
LG_VEHICLE_TYPE_CUSTOM = LG_F_CUSTOM
# Baseline
LG_VEHICLE_TYPE_AVG_AMOUNT_BY_LEVEL_AGGR = LG_F_AVG_AMOUNT_BY_LEVEL_AGGR
LG_VEHICLE_TYPE_AVG_AMOUNT_BY_LEVEL_DISAG = LG_F_AVG_AMOUNT_BY_LEVEL_DISAG
LG_VEHICLE_TYPE_AMOUNT_BY_WAREHOUSE_DISAG = LG_F_AMOUNT_BY_WAREHOUSE_DISAG
LG_VEHICLE_TYPE_UNIT_COST = LG_F_UNIT_COST
LG_VEHICLE_TYPE_WORKING_LIFE_YEARS = LG_F_WORKING_LIFE_YEARS  # 1d list of int; years other than base year appear in Targets section (for Vary number of trips per year option)
LG_VEHICLE_TYPE_FUEL_CONSUMPTION = "fuelConsumption"  # int
LG_VEHICLE_TYPE_REP_MAINT_COST_PERC_CAP_COST = "repairMaintAsPercCapitalCost"  # int
LG_VEHICLE_TYPE_NUM_TRIPS = "numTrips"  # 1d list of int; years other than base year appear in Targets section (for Vary number of trips per year option)
LG_VEHICLE_TYPE_AVG_KM_PER_TRIP = "avgKmPerTrip"  # 1d list of int; years other than base year appear in Targets section (for Vary number of trips per year option)
LG_VEHICLE_TYPE_COST_PETROL_PER_VEHIC_YEAR = "costPetrolPerVehicleYear"  # int
# Target setting
LG_VEHICLE_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR = LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR
LG_VEHICLE_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG = LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG
LG_VEHICLE_TYPE_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG = LG_F_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG
LG_VEHICLE_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR = LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR
LG_VEHICLE_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG = LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG
LG_VEHICLE_TYPE_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG = LG_F_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG
LG_VEHICLE_TYPE_TARG_EXISTING_PLANS_BY_LEVEL_AGGR = LG_F_TARG_EXISTING_PLANS_BY_LEVEL_AGGR
LG_VEHICLE_TYPE_TARG_EXISTING_PLANS_BY_LEVEL_DISAG = LG_F_TARG_EXISTING_PLANS_BY_LEVEL_DISAG
LG_VEHICLE_TYPE_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG = LG_F_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG
# Volume analytics
LG_VEHICLE_TYPE_PERC_SERVICED_BY_LEVEL_AGGR = "percServicedByLevelAggr"
LG_VEHICLE_TYPE_PERC_SERVICED_BY_WAREHOUSE_DISAG = "percServicedByWarehouseDisag"
LG_VEHICLE_TYPE_PERC_SERVICED_BY_LEVEL_DISAG = "percServicedByLevelDisag"
LG_VEHICLE_TYPE_DWELL_TIME_BY_LEVEL_AGGR = "dwellTimeByLevelAggr"
LG_VEHICLE_TYPE_DWELL_TIME_BY_WAREHOUSE_DISAG = "dwellTimeByWarehouseDisag"
LG_VEHICLE_TYPE_DWELL_TIME_BY_LEVEL_DISAG = "dwellTimeByLevelDisag"
LG_VEHICLE_TYPE_CAPACITY = LG_F_CAPACITY
LG_VEHICLE_TYPE_AVG_SPEED = "avgSpeed"
LG_VEHICLE_TYPE_MAX_DAYS_ON_ROUTE = "maxDaysOnRoute"


# =========================
# Worker records
# =========================
LG_WORKER_TYPE_ID = LG_F_WORKER_TYPE_ID  # string, int for defaults, GUID for custom.
LG_WORKER_TYPE_NAME = "name"  # string
LG_WORKER_TYPE_STR_CONST = "stringConstant"  # string;  Defaults only.
LG_WORKER_TYPE_CUSTOM = "custom"  # bool
# Baseline
LG_WORKER_TYPE_AVG_AMOUNT_BY_LEVEL_AGGR = LG_F_AVG_AMOUNT_BY_LEVEL_AGGR
LG_WORKER_TYPE_AVG_AMOUNT_BY_LEVEL_DISAG = LG_F_AVG_AMOUNT_BY_LEVEL_DISAG
LG_WORKER_TYPE_AMOUNT_BY_WAREHOUSE_DISAG = LG_F_AMOUNT_BY_WAREHOUSE_DISAG
LG_WORKER_TYPE_ANNUAL_WAGES = "annualWages"  # float
LG_WORKER_TYPE_ANNUAL_REAL_SALARY_INC = "annualRealSalaryIncrease"  # int
LG_WORKER_TYPE_ANNUAL_BENEFITS = "annualBenefits"  # float
LG_WORKER_TYPE_ANNUAL_BENEFITS_ADJUST = "annualBenefitsAdjustment"  # int
# Target setting
LG_WORKER_TYPE_TARG_CONFIG_OPTION = "targetConfigOption"  # int
LG_WORKER_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR = LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR
LG_WORKER_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG = LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG
LG_WORKER_TYPE_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG = LG_F_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG
LG_WORKER_TYPE_NUM_EXISTING_PLANS_BY_LEVEL_AGGR = LG_F_TARG_EXISTING_PLANS_BY_LEVEL_AGGR
LG_WORKER_TYPE_NUM_EXISTING_PLANS_BY_LEVEL_DISAG = LG_F_TARG_EXISTING_PLANS_BY_LEVEL_DISAG
LG_WORKER_TYPE_NUM_EXISTING_PLANS_BY_WAREHOUSE_DISAG = LG_F_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG
LG_WORKER_TYPE_VEHICLE_STD = (
    "targetNumWorkersPerVehicleType"  # 1d list of int; by vehicle type (if vehicle standard chosen)
)
LG_WORKER_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR = LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR
LG_WORKER_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG = LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG
LG_WORKER_TYPE_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG = LG_F_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG
# Volume analytics
LG_WORKER_TYPE_CLASSIFICATION = "classification"
LG_WORKER_TYPE_CAPACITY = LG_F_CAPACITY


# =========================
# Cold chain records
# =========================
LG_COLD_CHAIN_ID = LG_F_COLD_CHAIN_ID
LG_COLD_CHAIN_NAME = LG_F_NAME
LG_COLD_CHAIN_STR_CONST = LG_F_STR_CONST
LG_COLD_CHAIN_CUSTOM = LG_F_CUSTOM
LG_COLD_CHAIN_TYPE = "coldChainType"  # int
# Baseline
LG_COLD_CHAIN_AVG_AMOUNT_BY_LEVEL_AGGR = LG_F_AVG_AMOUNT_BY_LEVEL_AGGR
LG_COLD_CHAIN_AVG_AMOUNT_BY_LEVEL_DISAG = LG_F_AVG_AMOUNT_BY_LEVEL_DISAG
LG_COLD_CHAIN_AMOUNT_BY_WAREHOUSE_DISAG = LG_F_AMOUNT_BY_WAREHOUSE_DISAG
LG_COLD_CHAIN_UNIT_COST = LG_F_UNIT_COST
LG_COLD_CHAIN_CAPACITY = "capacity"
LG_COLD_CHAIN_MAINT_COST_PERC_CAP_COST = "maintCostPerc"
LG_COLD_CHAIN_WORKING_LIFE_YEARS = LG_F_WORKING_LIFE_YEARS
# Target setting
LG_COLD_CHAIN_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR = LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR
LG_COLD_CHAIN_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG = LG_F_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG
LG_COLD_CHAIN_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG = LG_F_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG
LG_COLD_CHAIN_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR = LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR
LG_COLD_CHAIN_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG = LG_F_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG
LG_COLD_CHAIN_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG = LG_F_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG
LG_COLD_CHAIN_TARG_EXISTING_PLANS_BY_LEVEL_AGGR = LG_F_TARG_EXISTING_PLANS_BY_LEVEL_AGGR
LG_COLD_CHAIN_TARG_EXISTING_PLANS_BY_LEVEL_DISAG = LG_F_TARG_EXISTING_PLANS_BY_LEVEL_DISAG
LG_COLD_CHAIN_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG = LG_F_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG

# =========================
# Third party contract records
# =========================
LG_TPL_CONTRACT_ID = LG_F_THIRD_PARTY_LOG_CONTRACT_ID
LG_TPL_CONTRACT_NAME = LG_F_NAME
LG_TPL_CONTRACT_STR_CONST = LG_F_STR_CONST
LG_TPL_CONTRACT_CUSTOM = LG_F_CUSTOM
LG_TPL_CONTRACT_COST = "cost"  # 1d list of int by year



from SpectrumCommon.Const.FP import *

def GetFP_methodName(m):
    result = ''
    if m == FP_CondomFM:
        result = 'GB_stCondomFM'
    elif m == FP_CondomML:
        result = 'GB_stCondomML'
    elif m == FP_fster:
        result = 'FP_stFemaleSt'
    elif m == FP_IUDCopper:
        result = 'GB_stIUDCopper'
    elif m == FP_IUDLngIus:
        result = 'GB_stIUDLngIus' 
    elif m == FP_mster:
        result = 'FP_stMaleSt'
    elif m == FP_vag_barrier:
        result = 'FP_stVagBarr'
    elif m == FP_VFT:
        result = 'GB_stSpermicides'
    elif m == FP_withdrawal:
        result = 'FP_stWithdrawal'
    elif m == FP_other:
        result = 'FP_stOther'
    elif m == FP_LAM:
        result = 'FP_stLAM'
    elif m == FP_SDM:
        result = 'GB_stSDM'
    elif m == FP_inj3month:
        result = 'GB_st3MonthDepoProvera'
    elif m == FP_inj2month:
        result = 'GB_st2MonthNoristerat'
    elif m == FP_inj1month:
        result = 'GB_st1monthLunelle'
    elif m == FP_inj6month:
        result = 'GB_st6Month'
    elif m == FP_Uninject:
        result = 'GB_stUninject'
    elif m == FP_Implanon:
        result = 'GB_stImplanon3Years'
    elif m == FP_Jadelle:
        result = 'GB_stJadelle5years'
    elif m == FP_Sino_Implant:
        result = 'GB_stSinoImplant4years'
    elif m == FP_StandardPill:
        result = 'GB_stStandardDailyRegiment'
    elif m == FP_Progestin:
        result = 'GB_stProgestinOnly'
    elif m == FP_Peri_coital:
        result = 'GB_stPeriCoitalContraception'
    elif m == FP_PerAbst:
        result = 'FP_stRhythm'
    elif m == FP_Traditional:
        result = 'GB_stOtherTraditional'
    # else:
    #     result = ''
    return result


from SpectrumCommon.Const.GB.GBConst import GB_NOT_FOUND

#######################################################################################################
#                                                                                                     #
#                                       Country Defaults DB (CDDB)                                    #
#                                                                                                     #
#######################################################################################################

RP_ISO_NUMERIC_COUNTRY_CODE_TAG_CDDB        = '<ISO Numeric Country Code>'
RP_ISO_ALPHA_3_COUNTRY_CODE_TAG_CDDB        = '<ISO Alpha-3 Country Code>'
RP_COUNTRY_NAME_TAG_CDDB                    = '<Country Name>'
RP_CLASSIFICATION_TAG_CDDB                  = '<Classification>'
RP_REGION_TAG_CDDB                          = '<Region>'
RP_BASELINE_AVG_TEMP_TAG_CDDB               = '<Baseline Average Temperature Celcius>'
RP_EXCH_RATE_TAG_CDDB                       = '<Exchange Rate to USD>'
RP_HH_SIZE_TAG_CDDB                         = '<Household Size (Overall)>'
RP_PERC_URBAN_INFORMAL_TAG_CDDB             = '<Percent Urban Informal>'
RP_COST_F_STERILIZATION_TAG_CDDB            = '<Cost Female Sterilization>'
RP_COST_M_STERILIZATION_TAG_CDDB            = '<Cost Male Sterilization>'
RP_COST_IUD_TAG_CDDB                        = '<Cost IUD>'
RP_COST_IMPLANT_TAG_CDDB                    = '<Cost Implant>'
RP_COST_INJECTIONS_TAG_CDDB                 = '<Cost Injections>'
RP_COST_PILL_TAG_CDDB                       = '<Cost Pill>'
RP_COST_CONDOMS_TAG_CDDB                    = '<Cost Condoms>'
RP_COST_VAG_BARR_TAG_CDDB                   = '<Cost Vaginal Barrier>'
RP_COST_OTHER_MODERN_TAG_CDDB               = '<Cost Other Modern>'
RP_COST_INDIRECTS_TAG_CDDB                  = '<Indirects>'
RP_AGE_START_PRIM_TAG_CDDB                  = '<Age Start Primary>'
RP_AGE_END_PRIM_TAG_CDDB                    = '<Age End Primary>'
RP_AGE_START_SEC_TAG_CDDB                   = '<Age Start Secondary>'
RP_AGE_END_SEC_TAG_CDDB                     = '<Age End Secondary>'
RP_PERC_ENROLL_PRIM_TAG_CDDB                = '<Percent Enrollment (Primary)>'
RP_PRIM_ENROLL_FEMALES_TAG_CDDB             = '<Primary Enrollment (Females)>'
RP_PRIM_ENROLL_MALES_TAG_CDDB               = '<Primary Enrollment (Males)>'
RP_PERC_ENROLL_SEC_TAG_CDDB                 = '<Percent Enrollment (Secondary)>'
RP_SEC_ENROLL_FEMALES_TAG_CDDB              = '<Secondary Enrollment (Females)>'
RP_SEC_ENROLL_MALES_TAG_CDDB                = '<Secondary Enrollment (Males)>'
RP_TEACHER_RATIO_PRIM_TAG_CDDB              = '<Teacher Ratio (Primary)>'
RP_TEACHER_RATIO_SEC_TAG_CDDB               = '<Teacher Ratio (Secondary)>'
RP_PRIM_COSTS_TAG_CDDB                      = '<Primary Costs>'
RP_SEC_COSTS_TAG_CDDB                       = '<Secondary Costs>'
RP_PRIM_TEACHER_SALARY_TAG_CDDB             = '<Primary Teacher Salary>'
RP_SEC_TEACHER_SALARY_TAG_CDDB              = '<Secondary Teacher Salary>'
RP_SKILLED_BIRTHS_TAG_CDDB                  = '<Skilled Births>'
RP_ANC_4_PLUS_TAG_CDDB                      = '<ANC 4 Plus>'
RP_PROFS_TAG_CDDB                           = '<Profs>'
RP_DOCTOR_SALARY_PUBLIC_TAG_CDDB            = '<Doctor Salary Public>'
RP_NURSE_SALARY_PUBLIC_TAG_CDDB             = '<Nurse Salary Public>'
RP_SKILLED_BIRTH_COSTS_TAG_CDDB             = '<Skilled Birth Costs>'
RP_ANC_COSTS_TAG_CDDB                       = '<ANC Costs>'
RP_PERC_DOCTORS_TAG_CDDB                    = '<Percent Doctors>'
RP_PERC_NURSES_TAG_CDDB                     = '<Percent Nurses>'
RP_PERC_IMPROVED_WATER_SOURCE_TAG_CDDB      = '<Percent Improved Water Source>'
RP_PERC_IMPROVED_SANI_TAG_CDDB              = '<Percent Improved Sanitation>'
RP_CAP_COST_PP_IMPROVED_WATER_TAG_CDDB      = '<Capital Cost PP Improved Water>'
RP_CAP_COST_PP_IMPROVED_SANI_TAG_CDDB       = '<Capital Cost PP Improved Sanitation>'
RP_PERC_PEOPLE_PHASE_4_PLUS_TAG_CDDB        = '<Percent People in Phase 4 Plus>'
RP_SEVERE_INSECURITY_TAG_CDDB               = '<Severe Insecurity>'
RP_AREA_HARVESTED_TAG_CDDB                  = '<Area Harvested>'
RP_STAPLE_PRODUCTION_TAG_CDDB               = '<Staple Production (In Tons)>'
RP_DOMINANT_CROP_TAG_CDDB                   = '<Dominant Crop (Maize, Wheat or Rice)>'
RP_AGRICULTURAL_LAND_TAG_CDDB               = '<Agricultural Land (Hectares)>'
RP_CROPLAND_HECTARES_TAG_CDDB               = '<Cropland Hectares>'
RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_TAG_CDDB   = '<Staple Annual Consumption per Capita (kg)>'
RP_PROD_COST_PER_TON_DOMINANT_CROP_TAG_CDDB = '<Production Cost per Ton for Dominant Crop>'
RP_DAILY_COST_FOOD_AID_PER_PERSON_TAG_CDDB  = '<Daily Cost of Food Aid per Person>'
RP_PERC_POP_ELECTRICITY_TAG_CDDB            = '<Percent Pop Electricity>'
RP_ELECTRICITY_PROD_PER_CAP_TAG_CDDB        = '<Electricity Produced per Capita>'
RP_COST_PER_KWH_TAG_CDDB                    = '<Cost per KWh>'
RP_LCOE_0_PER_PERSON_PER_DAY_TAG_CDDB       = '<LCOE 0 per Person per Day>'
RP_CLEAN_COOKING_TAG_CDDB                   = '<Clean Cooking>'
RP_CLEAN_COOKING_CAP_COST_TAG_CDDB          = '<Clean Cooking Capital Cost>'
RP_COST_BUILD_URBAN_HOUSEHOLD_TAG_CDDB      = '<Cost to Build Urban Household>'
RP_UNEMPLOYMENT_TAG_CDDB                    = '<Unemployment>'
RP_GDP_PER_CAP_TAG_CDDB                     = '<GDP per Capita>'
RP_LABOR_MALES_TAG_CDDB                     = '<Labor Males>'
RP_LABOR_FEMALES_TAG_CDDB                   = '<Labor Females>'
RP_LABOR_BOTH_TAG_CDDB                      = '<Labor Both>'
RP_GDP_BOOST_PERC_CHANGE_CDR_TAG_CDDB       = '<GDP Boost per Percent Change in CDR>'
RP_YOUTH_NEED_ALL_TAG_CDDB                  = '<Youth NEET All>'
RP_YOUTH_NEED_MALE_TAG_CDDB                 = '<Youth NEET Male>'
RP_YOUTH_NEED_FEMALE_TAG_CDDB               = '<Youth NEET Female>'
RP_1_DEGREE_GDP_TAG_CDDB                    = '<1 Degree GDP>'
RP_1_DEGREE_STAPLE_TAG_CDDB                 = '<1 Degree Staple>'
RP_2_DEGREE_STAPLE_TAG_CDDB                 = '<2 Degree Staple>'
RP_3_DEGREE_STAPLE_TAG_CDDB                 = '<3 Degree Staple>'
RP_1_DEGREE_ON_ENERGY_TAG_CDDB              = '<1 Degree on Energy>'

RP_ISO_ALPHA_3_COUNTRY_CODE_KEY_CDDB        = 'ISO3AlphaCountryCode'
RP_COUNTRY_NAME_KEY_CDDB                    = 'countryName'
RP_CLASSIFICATION_KEY_CDDB                  = 'classification'
RP_REGION_KEY_CDDB                          = 'region'
RP_BASELINE_AVG_TEMP_KEY_CDDB               = 'baselineAvgTemp'
RP_EXCH_RATE_KEY_CDDB                       = 'exchangeRate'
RP_HH_SIZE_KEY_CDDB                         = 'HHSize'
RP_PERC_URBAN_INFORMAL_KEY_CDDB             = 'percUrbanInformal'
RP_COST_F_STERILIZATION_KEY_CDDB            = 'costFSterilization'
RP_COST_M_STERILIZATION_KEY_CDDB            = 'costMSterilization'
RP_COST_IUD_KEY_CDDB                        = 'costIUD'
RP_COST_IMPLANT_KEY_CDDB                    = 'costImplant'
RP_COST_INJECTIONS_KEY_CDDB                 = 'costInjections'
RP_COST_PILL_KEY_CDDB                       = 'costPill'
RP_COST_CONDOMS_KEY_CDDB                    = 'costCondoms'
RP_COST_VAG_BARR_KEY_CDDB                   = 'costVaginalBarrier'
RP_COST_OTHER_MODERN_KEY_CDDB               = 'costOtherModern'
RP_COST_INDIRECTS_KEY_CDDB                  = 'indirects'
RP_AGE_START_PRIM_KEY_CDDB                  = 'ageStartPrimary'
RP_AGE_END_PRIM_KEY_CDDB                    = 'ageEndPrimary'
RP_AGE_START_SEC_KEY_CDDB                   = 'ageStartSecondary'
RP_AGE_END_SEC_KEY_CDDB                     = 'ageEndSecondary'
RP_PERC_ENROLL_PRIM_KEY_CDDB                = 'percEnrollmentPrimary'
RP_PRIM_ENROLL_FEMALES_KEY_CDDB             = 'primaryEnrollmentFemales'
RP_PRIM_ENROLL_MALES_KEY_CDDB               = 'primaryEnrollmentMales'
RP_PERC_ENROLL_SEC_KEY_CDDB                 = 'percEnrollmentSecondary'
RP_SEC_ENROLL_FEMALES_KEY_CDDB              = 'secondaryEnrollmentFemales'
RP_SEC_ENROLL_MALES_KEY_CDDB                = 'secondaryEnrollmentMales'
RP_TEACHER_RATIO_PRIM_KEY_CDDB              = 'teacherRatioPrimary'
RP_TEACHER_RATIO_SEC_KEY_CDDB               = 'teacherRatioSecondary'
RP_PRIM_COSTS_KEY_CDDB                      = 'primaryCosts'
RP_SEC_COSTS_KEY_CDDB                       = 'secondaryCosts'
RP_PRIM_TEACHER_SALARY_KEY_CDDB             = 'primaryTeacherSalary'
RP_SEC_TEACHER_SALARY_KEY_CDDB              = 'secondaryTeacherSalary'
RP_SKILLED_BIRTHS_KEY_CDDB                  = 'skilledBirths'
RP_ANC_4_PLUS_KEY_CDDB                      = 'ANC4Plus'
RP_PROFS_KEY_CDDB                           = 'profs'
RP_DOCTOR_SALARY_PUBLIC_KEY_CDDB            = 'doctorSalaryPublic'
RP_NURSE_SALARY_PUBLIC_KEY_CDDB             = 'nurseSalaryPublic'
RP_SKILLED_BIRTH_COSTS_KEY_CDDB             = 'skilledBirthCosts'
RP_ANC_COSTS_KEY_CDDB                       = 'ANCCosts'
RP_PERC_DOCTORS_KEY_CDDB                    = 'percDoctors'
RP_PERC_NURSES_KEY_CDDB                     = 'percNurses'
RP_PERC_IMPROVED_WATER_SOURCE_KEY_CDDB      = 'percImprovedWaterSource'
RP_PERC_IMPROVED_SANI_KEY_CDDB              = 'percImprovedSanitation'
RP_CAP_COST_PP_IMPROVED_WATER_KEY_CDDB      = 'capitalCostPPImprovedWater'
RP_CAP_COST_PP_IMPROVED_SANI_KEY_CDDB       = 'capitalCostPPImprovedSanitation'
RP_PERC_PEOPLE_PHASE_4_PLUS_KEY_CDDB        = 'percPeoplePhase4Plus'
RP_SEVERE_INSECURITY_KEY_CDDB               = 'severeInsecurity'
RP_AREA_HARVESTED_KEY_CDDB                  = 'areaHarvested'
RP_STAPLE_PRODUCTION_KEY_CDDB               = 'stapleProduction'
RP_DOMINANT_CROP_KEY_CDDB                   = 'dominantCrop'
RP_AGRICULTURAL_LAND_KEY_CDDB               = 'agriculturalLand'
RP_CROPLAND_HECTARES_KEY_CDDB               = 'croplandHectares'
RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_KEY_CDDB   = 'stapleAnnualConsumptionPerCapita'
RP_PROD_COST_PER_TON_DOMINANT_CROP_KEY_CDDB = 'productionCostPerTonDominantCrop'
RP_DAILY_COST_FOOD_AID_PER_PERSON_KEY_CDDB  = 'dailyCostFoodAidPerPerson'
RP_PERC_POP_ELECTRICITY_KEY_CDDB            = 'percPopElectricity'
RP_ELECTRICITY_PROD_PER_CAP_KEY_CDDB        = 'electricityProducedPerCapita'
RP_COST_PER_KWH_KEY_CDDB                    = 'costPerKWh'
RP_LCOE_0_PER_PERSON_PER_DAY_KEY_CDDB       = 'LCOE0PerPersonPerDay'
RP_CLEAN_COOKING_KEY_CDDB                   = 'cleanCooking'
RP_CLEAN_COOKING_CAP_COST_KEY_CDDB          = 'cleanCookingCapitalCost'
RP_COST_BUILD_URBAN_HOUSEHOLD_KEY_CDDB      = 'costBuildUrbanHH'
RP_UNEMPLOYMENT_KEY_CDDB                    = 'unemployment'
RP_GDP_PER_CAP_KEY_CDDB                     = 'GDPPerCapita'
RP_LABOR_MALES_KEY_CDDB                     = 'laborMales'
RP_LABOR_FEMALES_KEY_CDDB                   = 'laborFemales'
RP_LABOR_BOTH_KEY_CDDB                      = 'laborBoth'
RP_GDP_BOOST_PERC_CHANGE_CDR_KEY_CDDB       = 'GDPBoostPercChangeCDR'
RP_YOUTH_NEET_ALL_KEY_CDDB                  = 'youthNEETAll'
RP_YOUTH_NEET_MALE_KEY_CDDB                 = 'youthNEETMale'
RP_YOUTH_NEET_FEMALE_KEY_CDDB               = 'youthNEETFemale'
RP_1_DEGREE_GDP_KEY_CDDB                    = 'oneDegreeGDP'
RP_1_DEGREE_STAPLE_KEY_CDDB                 = 'oneDegreeStaple'
RP_2_DEGREE_STAPLE_KEY_CDDB                 = 'twoDegreeStaple'
RP_3_DEGREE_STAPLE_KEY_CDDB                 = 'threeDegreeStaple'
RP_1_DEGREE_ON_ENERGY_KEY_CDDB              = 'oneDegreeEnergy'

RP_tag_fields_CDDB = {
    RP_ISO_NUMERIC_COUNTRY_CODE_TAG_CDDB        : GB_NOT_FOUND,
    RP_ISO_ALPHA_3_COUNTRY_CODE_TAG_CDDB        : RP_ISO_ALPHA_3_COUNTRY_CODE_KEY_CDDB,
    RP_COUNTRY_NAME_TAG_CDDB                    : RP_COUNTRY_NAME_KEY_CDDB,
    RP_CLASSIFICATION_TAG_CDDB                  : RP_CLASSIFICATION_KEY_CDDB,
    RP_REGION_TAG_CDDB                          : RP_REGION_KEY_CDDB,
    RP_BASELINE_AVG_TEMP_TAG_CDDB               : RP_BASELINE_AVG_TEMP_KEY_CDDB,
    RP_EXCH_RATE_TAG_CDDB                       : RP_EXCH_RATE_KEY_CDDB,
    RP_HH_SIZE_TAG_CDDB                         : RP_HH_SIZE_KEY_CDDB,
    RP_PERC_URBAN_INFORMAL_TAG_CDDB             : RP_PERC_URBAN_INFORMAL_KEY_CDDB,
    RP_COST_F_STERILIZATION_TAG_CDDB            : RP_COST_F_STERILIZATION_KEY_CDDB,
    RP_COST_M_STERILIZATION_TAG_CDDB            : RP_COST_M_STERILIZATION_KEY_CDDB,
    RP_COST_IUD_TAG_CDDB                        : RP_COST_IUD_KEY_CDDB,
    RP_COST_IMPLANT_TAG_CDDB                    : RP_COST_IMPLANT_KEY_CDDB,
    RP_COST_INJECTIONS_TAG_CDDB                 : RP_COST_INJECTIONS_KEY_CDDB,
    RP_COST_PILL_TAG_CDDB                       : RP_COST_PILL_KEY_CDDB,
    RP_COST_CONDOMS_TAG_CDDB                    : RP_COST_CONDOMS_KEY_CDDB,
    RP_COST_VAG_BARR_TAG_CDDB                   : RP_COST_VAG_BARR_KEY_CDDB,
    RP_COST_OTHER_MODERN_TAG_CDDB               : RP_COST_OTHER_MODERN_KEY_CDDB,
    RP_COST_INDIRECTS_TAG_CDDB                  : RP_COST_INDIRECTS_KEY_CDDB,
    RP_AGE_START_PRIM_TAG_CDDB                  : RP_AGE_START_PRIM_KEY_CDDB,
    RP_AGE_END_PRIM_TAG_CDDB                    : RP_AGE_END_PRIM_KEY_CDDB,
    RP_AGE_START_SEC_TAG_CDDB                   : RP_AGE_START_SEC_KEY_CDDB,
    RP_AGE_END_SEC_TAG_CDDB                     : RP_AGE_END_SEC_KEY_CDDB,
    RP_PERC_ENROLL_PRIM_TAG_CDDB                : RP_PERC_ENROLL_PRIM_KEY_CDDB,
    RP_PRIM_ENROLL_FEMALES_TAG_CDDB             : RP_PRIM_ENROLL_FEMALES_KEY_CDDB,
    RP_PRIM_ENROLL_MALES_TAG_CDDB               : RP_PRIM_ENROLL_MALES_KEY_CDDB,
    RP_PERC_ENROLL_SEC_TAG_CDDB                 : RP_PERC_ENROLL_SEC_KEY_CDDB,
    RP_SEC_ENROLL_FEMALES_TAG_CDDB              : RP_SEC_ENROLL_FEMALES_KEY_CDDB,
    RP_SEC_ENROLL_MALES_TAG_CDDB                : RP_SEC_ENROLL_MALES_KEY_CDDB,
    RP_TEACHER_RATIO_PRIM_TAG_CDDB              : RP_TEACHER_RATIO_PRIM_KEY_CDDB,
    RP_TEACHER_RATIO_SEC_TAG_CDDB               : RP_TEACHER_RATIO_SEC_KEY_CDDB,
    RP_PRIM_COSTS_TAG_CDDB                      : RP_PRIM_COSTS_KEY_CDDB,
    RP_SEC_COSTS_TAG_CDDB                       : RP_SEC_COSTS_KEY_CDDB,
    RP_PRIM_TEACHER_SALARY_TAG_CDDB             : RP_PRIM_TEACHER_SALARY_KEY_CDDB,
    RP_SEC_TEACHER_SALARY_TAG_CDDB              : RP_SEC_TEACHER_SALARY_KEY_CDDB,
    RP_SKILLED_BIRTHS_TAG_CDDB                  : RP_SKILLED_BIRTHS_KEY_CDDB,
    RP_ANC_4_PLUS_TAG_CDDB                      : RP_ANC_4_PLUS_KEY_CDDB,
    RP_PROFS_TAG_CDDB                           : RP_PROFS_KEY_CDDB,
    RP_DOCTOR_SALARY_PUBLIC_TAG_CDDB            : RP_DOCTOR_SALARY_PUBLIC_KEY_CDDB,
    RP_NURSE_SALARY_PUBLIC_TAG_CDDB             : RP_NURSE_SALARY_PUBLIC_KEY_CDDB,
    RP_SKILLED_BIRTH_COSTS_TAG_CDDB             : RP_SKILLED_BIRTH_COSTS_KEY_CDDB,
    RP_ANC_COSTS_TAG_CDDB                       : RP_ANC_COSTS_KEY_CDDB,
    RP_PERC_DOCTORS_TAG_CDDB                    : RP_PERC_DOCTORS_KEY_CDDB,
    RP_PERC_NURSES_TAG_CDDB                     : RP_PERC_NURSES_KEY_CDDB,
    RP_PERC_IMPROVED_WATER_SOURCE_TAG_CDDB      : RP_PERC_IMPROVED_WATER_SOURCE_KEY_CDDB,
    RP_PERC_IMPROVED_SANI_TAG_CDDB              : RP_PERC_IMPROVED_SANI_KEY_CDDB,
    RP_CAP_COST_PP_IMPROVED_WATER_TAG_CDDB      : RP_CAP_COST_PP_IMPROVED_WATER_KEY_CDDB,
    RP_CAP_COST_PP_IMPROVED_SANI_TAG_CDDB       : RP_CAP_COST_PP_IMPROVED_SANI_KEY_CDDB,
    RP_PERC_PEOPLE_PHASE_4_PLUS_TAG_CDDB        : RP_PERC_PEOPLE_PHASE_4_PLUS_KEY_CDDB,
    RP_SEVERE_INSECURITY_TAG_CDDB               : RP_SEVERE_INSECURITY_KEY_CDDB,
    RP_AREA_HARVESTED_TAG_CDDB                  : RP_AREA_HARVESTED_KEY_CDDB,
    RP_STAPLE_PRODUCTION_TAG_CDDB               : RP_STAPLE_PRODUCTION_KEY_CDDB,
    RP_DOMINANT_CROP_TAG_CDDB                   : RP_DOMINANT_CROP_KEY_CDDB,
    RP_AGRICULTURAL_LAND_TAG_CDDB               : RP_AGRICULTURAL_LAND_KEY_CDDB,
    RP_CROPLAND_HECTARES_TAG_CDDB               : RP_CROPLAND_HECTARES_KEY_CDDB,
    RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_TAG_CDDB   : RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_KEY_CDDB,
    RP_PROD_COST_PER_TON_DOMINANT_CROP_TAG_CDDB : RP_PROD_COST_PER_TON_DOMINANT_CROP_KEY_CDDB,
    RP_DAILY_COST_FOOD_AID_PER_PERSON_TAG_CDDB  : RP_DAILY_COST_FOOD_AID_PER_PERSON_KEY_CDDB,
    RP_PERC_POP_ELECTRICITY_TAG_CDDB            : RP_PERC_POP_ELECTRICITY_KEY_CDDB,
    RP_ELECTRICITY_PROD_PER_CAP_TAG_CDDB        : RP_ELECTRICITY_PROD_PER_CAP_KEY_CDDB,
    RP_COST_PER_KWH_TAG_CDDB                    : RP_COST_PER_KWH_KEY_CDDB,
    RP_LCOE_0_PER_PERSON_PER_DAY_TAG_CDDB       : RP_LCOE_0_PER_PERSON_PER_DAY_KEY_CDDB,
    RP_CLEAN_COOKING_TAG_CDDB                   : RP_CLEAN_COOKING_KEY_CDDB,
    RP_CLEAN_COOKING_CAP_COST_TAG_CDDB          : RP_CLEAN_COOKING_CAP_COST_KEY_CDDB,
    RP_COST_BUILD_URBAN_HOUSEHOLD_TAG_CDDB      : RP_COST_BUILD_URBAN_HOUSEHOLD_KEY_CDDB,
    RP_UNEMPLOYMENT_TAG_CDDB                    : RP_UNEMPLOYMENT_KEY_CDDB,
    RP_GDP_PER_CAP_TAG_CDDB                     : RP_GDP_PER_CAP_KEY_CDDB,
    RP_LABOR_MALES_TAG_CDDB                     : RP_LABOR_MALES_KEY_CDDB,
    RP_LABOR_FEMALES_TAG_CDDB                   : RP_LABOR_FEMALES_KEY_CDDB,
    RP_LABOR_BOTH_TAG_CDDB                      : RP_LABOR_BOTH_KEY_CDDB,
    RP_GDP_BOOST_PERC_CHANGE_CDR_TAG_CDDB       : RP_GDP_BOOST_PERC_CHANGE_CDR_KEY_CDDB,
    RP_YOUTH_NEED_ALL_TAG_CDDB                  : RP_YOUTH_NEET_ALL_KEY_CDDB,
    RP_YOUTH_NEED_MALE_TAG_CDDB                 : RP_YOUTH_NEET_MALE_KEY_CDDB,
    RP_YOUTH_NEED_FEMALE_TAG_CDDB               : RP_YOUTH_NEET_FEMALE_KEY_CDDB,
    RP_1_DEGREE_GDP_TAG_CDDB                    : RP_1_DEGREE_GDP_KEY_CDDB,
    RP_1_DEGREE_STAPLE_TAG_CDDB                 : RP_1_DEGREE_STAPLE_KEY_CDDB,
    RP_2_DEGREE_STAPLE_TAG_CDDB                 : RP_2_DEGREE_STAPLE_KEY_CDDB,
    RP_3_DEGREE_STAPLE_TAG_CDDB                 : RP_3_DEGREE_STAPLE_KEY_CDDB,
    RP_1_DEGREE_ON_ENERGY_TAG_CDDB              : RP_1_DEGREE_ON_ENERGY_KEY_CDDB,
}

RP_tag_columns_CDDB = {
    RP_ISO_NUMERIC_COUNTRY_CODE_TAG_CDDB        : 1,
    RP_ISO_ALPHA_3_COUNTRY_CODE_TAG_CDDB        : 2,
    RP_COUNTRY_NAME_TAG_CDDB                    : 3,
    RP_CLASSIFICATION_TAG_CDDB                  : GB_NOT_FOUND,
    RP_REGION_TAG_CDDB                          : GB_NOT_FOUND,
    RP_BASELINE_AVG_TEMP_TAG_CDDB               : GB_NOT_FOUND,
    RP_EXCH_RATE_TAG_CDDB                       : GB_NOT_FOUND,
    RP_HH_SIZE_TAG_CDDB                         : GB_NOT_FOUND,
    RP_PERC_URBAN_INFORMAL_TAG_CDDB             : GB_NOT_FOUND,
    RP_COST_F_STERILIZATION_TAG_CDDB            : GB_NOT_FOUND,
    RP_COST_M_STERILIZATION_TAG_CDDB            : GB_NOT_FOUND,
    RP_COST_IUD_TAG_CDDB                        : GB_NOT_FOUND,
    RP_COST_IMPLANT_TAG_CDDB                    : GB_NOT_FOUND,
    RP_COST_INJECTIONS_TAG_CDDB                 : GB_NOT_FOUND,
    RP_COST_PILL_TAG_CDDB                       : GB_NOT_FOUND,
    RP_COST_CONDOMS_TAG_CDDB                    : GB_NOT_FOUND,
    RP_COST_VAG_BARR_TAG_CDDB                   : GB_NOT_FOUND,
    RP_COST_OTHER_MODERN_TAG_CDDB               : GB_NOT_FOUND,
    RP_COST_INDIRECTS_TAG_CDDB                  : GB_NOT_FOUND,
    RP_AGE_START_PRIM_TAG_CDDB                  : GB_NOT_FOUND,
    RP_AGE_END_PRIM_TAG_CDDB                    : GB_NOT_FOUND,
    RP_AGE_START_SEC_TAG_CDDB                   : GB_NOT_FOUND,
    RP_AGE_END_SEC_TAG_CDDB                     : GB_NOT_FOUND,
    RP_PERC_ENROLL_PRIM_TAG_CDDB                : GB_NOT_FOUND,
    RP_PRIM_ENROLL_FEMALES_TAG_CDDB             : GB_NOT_FOUND,
    RP_PRIM_ENROLL_MALES_TAG_CDDB               : GB_NOT_FOUND,
    RP_PERC_ENROLL_SEC_TAG_CDDB                 : GB_NOT_FOUND,
    RP_SEC_ENROLL_FEMALES_TAG_CDDB              : GB_NOT_FOUND,
    RP_SEC_ENROLL_MALES_TAG_CDDB                : GB_NOT_FOUND,
    RP_TEACHER_RATIO_PRIM_TAG_CDDB              : GB_NOT_FOUND,
    RP_TEACHER_RATIO_SEC_TAG_CDDB               : GB_NOT_FOUND,
    RP_PRIM_COSTS_TAG_CDDB                      : GB_NOT_FOUND,
    RP_SEC_COSTS_TAG_CDDB                       : GB_NOT_FOUND,
    RP_PRIM_TEACHER_SALARY_TAG_CDDB             : GB_NOT_FOUND,
    RP_SEC_TEACHER_SALARY_TAG_CDDB              : GB_NOT_FOUND,
    RP_SKILLED_BIRTHS_TAG_CDDB                  : GB_NOT_FOUND,
    RP_ANC_4_PLUS_TAG_CDDB                      : GB_NOT_FOUND,
    RP_PROFS_TAG_CDDB                           : GB_NOT_FOUND,
    RP_DOCTOR_SALARY_PUBLIC_TAG_CDDB            : GB_NOT_FOUND,
    RP_NURSE_SALARY_PUBLIC_TAG_CDDB             : GB_NOT_FOUND,
    RP_SKILLED_BIRTH_COSTS_TAG_CDDB             : GB_NOT_FOUND,
    RP_ANC_COSTS_TAG_CDDB                       : GB_NOT_FOUND,
    RP_PERC_DOCTORS_TAG_CDDB                    : GB_NOT_FOUND,
    RP_PERC_NURSES_TAG_CDDB                     : GB_NOT_FOUND,
    RP_PERC_IMPROVED_WATER_SOURCE_TAG_CDDB      : GB_NOT_FOUND,
    RP_PERC_IMPROVED_SANI_TAG_CDDB              : GB_NOT_FOUND,
    RP_CAP_COST_PP_IMPROVED_WATER_TAG_CDDB      : GB_NOT_FOUND,
    RP_CAP_COST_PP_IMPROVED_SANI_TAG_CDDB       : GB_NOT_FOUND,
    RP_PERC_PEOPLE_PHASE_4_PLUS_TAG_CDDB        : GB_NOT_FOUND,
    RP_SEVERE_INSECURITY_TAG_CDDB               : GB_NOT_FOUND,
    RP_AREA_HARVESTED_TAG_CDDB                  : GB_NOT_FOUND,
    RP_STAPLE_PRODUCTION_TAG_CDDB               : GB_NOT_FOUND,
    RP_DOMINANT_CROP_TAG_CDDB                   : GB_NOT_FOUND,
    RP_AGRICULTURAL_LAND_TAG_CDDB               : GB_NOT_FOUND,
    RP_CROPLAND_HECTARES_TAG_CDDB               : GB_NOT_FOUND,
    RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_TAG_CDDB   : GB_NOT_FOUND,
    RP_PROD_COST_PER_TON_DOMINANT_CROP_TAG_CDDB : GB_NOT_FOUND,
    RP_DAILY_COST_FOOD_AID_PER_PERSON_TAG_CDDB  : GB_NOT_FOUND,
    RP_PERC_POP_ELECTRICITY_TAG_CDDB            : GB_NOT_FOUND,
    RP_ELECTRICITY_PROD_PER_CAP_TAG_CDDB        : GB_NOT_FOUND,
    RP_COST_PER_KWH_TAG_CDDB                    : GB_NOT_FOUND,
    RP_LCOE_0_PER_PERSON_PER_DAY_TAG_CDDB       : GB_NOT_FOUND,
    RP_CLEAN_COOKING_TAG_CDDB                   : GB_NOT_FOUND,
    RP_CLEAN_COOKING_CAP_COST_TAG_CDDB          : GB_NOT_FOUND,
    RP_COST_BUILD_URBAN_HOUSEHOLD_TAG_CDDB      : GB_NOT_FOUND,
    RP_UNEMPLOYMENT_TAG_CDDB                    : GB_NOT_FOUND,
    RP_GDP_PER_CAP_TAG_CDDB                     : GB_NOT_FOUND,
    RP_LABOR_MALES_TAG_CDDB                     : GB_NOT_FOUND,
    RP_LABOR_FEMALES_TAG_CDDB                   : GB_NOT_FOUND,
    RP_LABOR_BOTH_TAG_CDDB                      : GB_NOT_FOUND,
    RP_GDP_BOOST_PERC_CHANGE_CDR_TAG_CDDB       : GB_NOT_FOUND,
    RP_YOUTH_NEED_ALL_TAG_CDDB                  : GB_NOT_FOUND,
    RP_YOUTH_NEED_MALE_TAG_CDDB                 : GB_NOT_FOUND,
    RP_YOUTH_NEED_FEMALE_TAG_CDDB               : GB_NOT_FOUND,
    RP_1_DEGREE_GDP_TAG_CDDB                    : GB_NOT_FOUND,
    RP_1_DEGREE_STAPLE_TAG_CDDB                 : GB_NOT_FOUND,
    RP_2_DEGREE_STAPLE_TAG_CDDB                 : GB_NOT_FOUND,
    RP_3_DEGREE_STAPLE_TAG_CDDB                 : GB_NOT_FOUND,
    RP_1_DEGREE_ON_ENERGY_TAG_CDDB              : GB_NOT_FOUND,
}

#######################################################################################################
#                                                                                                     #
#                                       Contraceptive Use DBs (CUDB)                                  #
#                                                                                                     #
#######################################################################################################

RP_MEDIAN_DB      = 1
RP_ACCELERATED_DB = 2

RP_ISO_NUMERIC_COUNTRY_CODE_DB_TAG_CUDB        = '<ISO Numeric Country Code>'
RP_ISO_ALPHA_3_COUNTRY_CODE_DB_TAG_CUDB        = '<ISO Alpha-3 Country Code>'
RP_COUNTRY_NAME_DB_TAG_CUDB                    = '<Country Name>'

RP_ISO_ALPHA_3_COUNTRY_CODE_KEY_CUDB    = 'ISO3AlphaCountryCode'
RP_COUNTRY_NAME_KEY_UDDB                = 'countryName'
RP_CONTRACEPTIVE_USE_KEY_CUDB           = 'contraceptiveUse'

RP_FIRST_YEAR_CUDB = 2020
RP_FINAL_YEAR_CUDB = 2075

#######################################################################################################
#                                                                                                     #
#                                       SSP DBs (SSPDB)                                  #
#                                                                                                     #
#######################################################################################################

RP_SSP1_DB = 1
RP_SSP2_DB = 2
RP_SSP3_DB = 3
RP_SSP5_DB = 4

RP_ISO_NUMERIC_COUNTRY_CODE_DB_TAG_SSPDB  = '<ISO Numeric Country Code>'
RP_ISO_ALPHA_3_COUNTRY_CODE_DB_TAG_SSPDB  = '<ISO Alpha-3 Country Code>'
RP_COUNTRY_NAME_DB_TAG_SSPDB              = '<Country Name>'

RP_ISO_ALPHA_3_COUNTRY_CODE_KEY_SSPDB     = 'ISO3AlphaCountryCode'
RP_COUNTRY_NAME_KEY_SSPDB                 = 'countryName'
RP_BASELINE_AVG_TEMP_KEY_SSPDB            = 'baselineAvgTemp'

RP_FIRST_YEAR_SSPDB = 2020
RP_FINAL_YEAR_SSPDB = 2075

#######################################################################################################
#                                                                                                     #
#                                       Urban DB (UDB)                                                #
#                                                                                                     #
#######################################################################################################

RP_ISO_NUMERIC_COUNTRY_CODE_DB_TAG_UDB  = '<ISO Numeric Country Code>'
RP_ISO_ALPHA_3_COUNTRY_CODE_DB_TAG_UDB  = '<ISO Alpha-3 Country Code>'
RP_COUNTRY_NAME_DB_TAG_UDB              = '<Country Name>'

RP_ISO_ALPHA_3_COUNTRY_CODE_KEY_UDB     = 'ISO3AlphaCountryCode'
RP_COUNTRY_NAME_KEY_UDB                 = 'countryName'
RP_PERC_POP_LIVING_URBAN_AREAS_KEY_UDB  = 'percPopLivingInUrbanAreas'

RP_FIRST_YEAR_UDB = 2020
RP_FINAL_YEAR_UDB = 2075


from SpectrumCommon.Const.FP import *
from SpectrumCommon.Const.GB import *
from SpectrumCommon.Modvars.GB.GBDefs import *
from SpectrumCommon.Modvars.FP.FPIndices import *


fp_age_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['fp_age',  gb_year_dim], 
                                                  indices=[fp_age_indices, gb_year_indices], **kwargs)

fp_age_method_need_year_shape = lambda d, **kwargs : gb_shape(d,'4D float',
                                                              dim=['fp_age', 'fp_method', 'fp_need', gb_year_dim], 
                                                              indices=[fp_age_indices, fp_method_indices, fp_need_indices, gb_year_indices], **kwargs)


fp_age_need_year_shape = lambda d, **kwargs : gb_shape(d,'3D float',
                                                       dim=['fp_age', 'fp_need', gb_year_dim], 
                                                       indices=[fp_age_indices, fp_need_indices, gb_year_indices], **kwargs)

fp_need_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                   dim=['fp_need', gb_year_dim], 
                                                   indices=[fp_need_indices, gb_year_indices], **kwargs)

fp_age_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                  dim=['fp_age', gb_year_dim], 
                                                  indices=[fp_age_indices, gb_year_indices], **kwargs)

fp_age_shape = lambda d, **kwargs : gb_shape(d,'1D float',
                                             dim=['fp_age'], 
                                             indices=[fp_age_indices], **kwargs)



fp_method_set_shape = lambda d, **kwargs : gb_shape(d,'1D int',
                                                    options=fp_method_indices, **kwargs)

fp_age_method_year_shape = lambda d, **kwargs : gb_shape(d,'3D float',
                                                         dim=['fp_age', 'fp_method',  gb_year_dim], 
                                                         indices=[fp_age_indices,fp_method_indices,gb_year_indices], **kwargs)

fp_method_year_shape = lambda d, **kwargs : gb_shape(d,'2D float',
                                                     dim=['fp_method', gb_year_dim], 
                                                     indices=[fp_method_indices,gb_year_indices], **kwargs)

fp_method_source_year_shape = lambda d, **kwargs : gb_shape(d,'3D float',
                                                              dim=['fp_method', 'fp_source', gb_year_dim], 
                                                              indices=[fp_method_indices, fp_need_indices, gb_year_indices], **kwargs)

fp_method_age_source_year_shape = lambda d, **kwargs : gb_shape(d,'4D float',
                                                              dim=['fp_method', 'fp_age', 'fp_source', gb_year_dim], 
                                                              indices=[fp_method_indices, fp_age_indices, fp_need_indices, gb_year_indices], **kwargs)

FP_modvar_shapes = {
        # db,
        #General
        FP_ActiveMethodsTag                : gb_shape("Active methods selected in the configuration", 
                                                      '1D bool',
                                                      dim=['fp_method'],
                                                      range='True, False',
                                                      indices=[fp_method_indices], 
                                                      note="Atleast one method needs to be active"),
        FP_AgeGroupOptionTag               : gb_int_shape('Selected age group option', 
                                                           options={'Single age group': FP_Single_Age_Group, 
                                                                    'Five year age groups':FP_Five_Year_Age_Groups},
                                                            has_default=False) ,
        FP_GoalTag                         : gb_int_shape('Selected Famplan goal', 
                                                           options={'Unmet need goal'          : FP_Unmet_need,
                                                                    'Desired TFR goal'         : FP_Desired_TFR,
                                                                    'Prevalence goal(default)' : FP_Prevalence,
                                                                    'TFR goal'                 : FP_TFR,
                                                                    'Modern CPR goal'          : FP_mCPR ,
                                                                    'Additional modern goal'   : FP_AddMdrnMeth,
                                                                    'Pregnancy goal'           : FP_Pregnancy },
                                                            has_default=False),   


        FP_AbortionOptionTag               : gb_int_shape('Abortion option flag', 
                                                           options={'Calculate abortions':FP_calculate_abortions, 
                                                                    'Specify TAR':FP_specify_TAR},
                                                           has_default=False,
                                                           defaultDataTags=['fp_easydata']) ,
        FP_NeedOptionTag                   : gb_bool_shape('Need option',has_default=False),

        FP_SourceMixTag                    : fp_method_source_year_shape("Source mix", 
                                                                         range='0-100', 
                                                                         note="Values must sum to 100 accross the method dimension"),
        FP_MethodMixTag                    : fp_age_method_need_year_shape("Method mix", 
                                                                           range='0-100', 
                                                                           note="Values must sum to 100 accross the method dimension",
                                                                           defaultDataTags=['fp_easydata']),
        FP_NumSourcesTag                   : gb_int_shape('Number of sources'),
        FP_NumNeedsTag                     : gb_int_shape('Number of needs'),
        FP_NumCustomMethodsTag             : gb_int_shape('Number of custom created contraceptive methods'),
        FP_PregnancyRateTag                : gb_float_shape('Pregnancy Rate', range='0-1.0'),
        FP_PercentPregWithMiscarriageTag   : gb_float_shape('Percent pregnant with miscarriage', range='0-1.0'),
        FP_PropUnintendedPregEndInAbortTag : gb_float_shape('Proportion of unintended pregnancies ending in abortion', defaultDataTags=['fp_easydata']),
        FP_PropUnsafeAbortTag              : gb_float_shape('Proportion of unsafe abortions', defaultDataTags=['fp_easydata']),
        FP_MortalityRateSafeAbortTag       : gb_float_shape('Mortality rate of safe abortions'),
        FP_MortalityRateUnsafeAbortTag     : gb_float_shape('Mortality rate of unsafe abortions', defaultDataTags=['fp_easydata']),
        FP_PropPregWantedLaterTag          : gb_float_shape('Proportion of pregnancies wanted later'),
        FP_PropPregUnwantedTag             : gb_float_shape('Proportion of unwanted pregnancies'),
        FP_MiscarriagePercentTag           : gb_int_shape('Miscarriage percent', range='0-100'),
        FP_CalcMethOptionTag               : gb_int_shape('Calculation method option', options={                                                                      
                                                                            'Use total fecundity and proximate determinants' : FP_TotFec,
                                                                            'Use pregnancy rate approach' : FP_PregRate 
                                                                        }), #,
        FP_MethodPermTermTag               : fp_method_set_shape('Permanent contraceptive methods'),
        FP_MethodLongTermTag               : fp_method_set_shape('Long term contraceptive methods'),
        FP_MethodshortTermTag              : fp_method_set_shape('Short term contraceptive methods'),
        FP_EffectivenessTag                : fp_age_method_year_shape('Units per CYP', defaultDataTags=['fp_effectiveness_db']),
        FP_UnitsPerCYPTag                  : fp_method_year_shape('Units per CYP'),
        FP_AverageAgeTag                   : fp_method_year_shape('Average age'),
        FP_DurationOfUseTag                : fp_method_year_shape('Duration of use'),
        FP_SourceMixTag                    : fp_method_source_year_shape("Source mix", 
                                                                         range='0-100', 
                                                                         note="Values must sum to 100 accross the method dimension"),
        FP_CostPerUATag                    : fp_method_source_year_shape("Cost per UA"),
        FP_FeesTag                         : fp_method_source_year_shape("Fees"),
        FP_GrossCostTag                    : fp_method_source_year_shape("Gross cost"),
        FP_RevenueTag                      : fp_method_source_year_shape("Revenue"),
        FP_NetCostTag                      : fp_method_source_year_shape("Net cost"),
        FP_CommoditiesTag                  : fp_method_source_year_shape("Commodities"),
        FP_PercentInUnionTag               : fp_age_year_shape('Percent in union', defaultDataTags=['fp_easydata']),
        FP_PPITag                          : fp_age_year_shape('PPI', defaultDataTags=['fp_easydata']),
        FP_TARTag                          : fp_age_year_shape('Total abortion rate (TAR)'),
        FP_AbortionPercentTag              : fp_age_year_shape('TAbortion percent'),
        FP_SterilityTag                    : fp_age_year_shape('Sterility'),
        FP_MarriageIndexTag                : fp_age_year_shape('Marriage index'), 
        FP_PPIIndexTag                     : fp_age_year_shape('PPI index'), 
        FP_AbortionIndexTag                : fp_age_year_shape('Abortion index'),
        FP_SterilityIndexTag               : fp_age_year_shape('Sterility index'),
        FP_PrevalenceIndexTag              : fp_age_year_shape('Prevalence index'),
        FP_TFRTag                          : fp_age_year_shape('Total fertility rate (TFR)', defaultDataTags=['fp_easydata']),
        FP_WRATag                          : fp_age_year_shape('Women of reproductive age (WRA)', defaultDataTags=['fp_easydata']),
        FP_MWRATag                         : fp_age_year_shape('Married women of reproductive age (MWRA)', defaultDataTags=['fp_easydata']),
        FP_MortalityRateTag                : fp_age_year_shape('Mortality rate'),
        FP_BirthsTag                       : fp_age_year_shape('Births'),
        FP_BirthsAvertedTag                : fp_age_year_shape('Births averted'),
        FP_AbortionsTag                    : fp_age_year_shape('Abortions'),
        FP_AverageEffectivenessTag         : fp_age_need_year_shape('Average effectiveness', defaultDataTags=['fp_effectiveness_db']),
        FP_PrevalenceTag                   : fp_age_need_year_shape('Contraception prevalence rate', defaultDataTags=['fp_easydata']), 
        FP_MPrevalenceTag                  : fp_age_need_year_shape('Modern prevalence'),
        FP_AddMdrnUsersPrevTag             : fp_age_need_year_shape('Additional modern users prevalence'),
        FP_UnmetNeedTag                    : fp_age_need_year_shape('Unmet need', defaultDataTags=['fp_easydata']),
        FP_UnmetNeedReductionTag           : fp_need_year_shape('Unmet need reduction'), 
        FP_WantedTFRTag                    : fp_age_shape('Wanted TFR'),
        FP_TFRdifferenceTag                : gb_year_shape('TFR differences'),
        FP_TestUnmetNeedFromPrevTag        : gb_bool_shape('Test unmet need from prev', has_default=False),
        FP_CalcCostRegressionTag           : gb_bool_shape('Calculate cost regression', has_default=False),
        FP_ASFRInput_TFRGoalTag            : gb_int_shape('ASFR input', 
                                                          options={                                                                      
                                                              'TFR MT TFR Goal' : FP_TFR_MT_TFRGOAL,
                                                              'ASFR TFR Goal' : FP_ASFR_TFRGOAL 
                                                          },
                                                          has_default=False),
        FP_ChildSurvInputsTag              : gb_shape('Child survival inputs','1D float',
                                                  dim=['fp_child_survival'], 
                                                  indices=[fp_child_survival_indices],
                                                  defaultDataTags=['fp_easydata']),

        FP_PACInputsTag                    : gb_shape('Pac inputs','1D float',
                                                  dim=['fp_pac_inputs', gb_year_dim], 
                                                  indices=[fp_pac_inputs_indices, gb_year_indices]),
        FP_LegalAbortNeedTreatTag          : gb_float_shape('Legal abortion needing treatment'),
        FP_PACOutputsTag                   : gb_shape('Pac outputs','1D float',
                                                  dim=['fp_pac_outputs', gb_year_dim], 
                                                  indices=[fp_pac_outputs_indices, gb_year_indices]),
        FP_LAMinsusceptibleTag             : gb_shape('LAM insusceptible','1D float',
                                                  dim=['fp_lam'], 
                                                  indices=[fp_lam_indices]),
        FP_EPSurveyYearTFRTag              : gb_float_shape('EP Survey Year TFR'),
        FP_TFTag                           : fp_age_year_shape('Total fecundity', defaultDataTags=['fp_easydata']),
        FP_PRTag                           : fp_age_shape('Pregnancy rate'),
        FP_DiscontinuationRateTag          : fp_method_year_shape('Discontinuation rate'),
        FP_UsersTag                        : fp_method_age_source_year_shape('Users'),
        FP_AcceptorsTag                    : fp_method_age_source_year_shape('Acceptors'),
        FP_GrowthRateTag                   : fp_method_age_source_year_shape('Growth rates'),
        FP_PregnanciesTag                  : gb_shape('Pac outputs','1D float',
                                                  dim=['fp_age', 'fp_pregnancies', gb_year_dim], 
                                                  indices=[fp_age_indices, fp_pregnancies_indices, gb_year_indices]),
        
        FP_ChildSurvOutputsTag             : gb_shape('Child survival outputs','2D float',
                                                  dim=['fp_child_survival', gb_year_dim], 
                                                  indices=[fp_child_survival_indices, gb_year_indices]),
        FP_PregAvertedTag                  : gb_year_shape('Pregnancies averted'),
        FP_MaternalDeathsAvertedTag        : gb_year_shape('Maternal deaths averted'),
        FP_UnsafeAbortionsAvertedTag       : gb_year_shape('Unsafe abortions averted averted'),
        FP_SourceTag                       : gb_shape('Source of Famplan data', 'dict', innerShape={
                                                'main' : gb_str_shape('Source of Famplan data'),
                                                'goal' : gb_str_shape('Source of Famplan data')
                                                },
                                                defaultDataTags=['fp_easydata']),
        FP_DataSourceTag                   : gb_str_shape('Source of Famplan data', defaultDataTags=['fp_easydata']),
        FP_AvgHouseholdTag                 : gb_int_shape('Average household size'), 
        FP_GoalAnnualChangeTag             : gb_shape('Annual change goal', 'dictList', innerShape={
                'useAnnualChange' : gb_bool_shape('Use annual change flag', has_default=False), 
                'annualChange' : gb_float_shape('Annual change value')} 
        ),
        FP_RelativeRisksTag                : gb_shape('Relative risk', 'dict', innerShape={
            'ageBirthOrder' : gb_shape("age and birth order", 
                                       "2D float", 
                                       dim=['fp_rr_birth_order', 'fp_rr_age_order'],
                                       indices=[fp_rr_birth_order_indices, fp_rr_age_order_indices]), 

            'birthInterval' : gb_shape("birth interval",
                                       "1D float",
                                       dim=['fp_rr_birth_interval'],
                                       indices=[fp_rr_birth_interval_indices]), 
            
            'ageBirthAdv'   : gb_shape("advance age and birth order",
                                       "3D float", 
                                       dim=['fp_rr_advance', 'fp_birth_order', 'fp_age_order'],
                                       indices=[fp_rr_advance_indices, fp_rr_birth_order_indices, fp_rr_age_order_indices]), 
            'birthIntAdv'   : gb_shape("advance birth interval",
                                       "2D float",
                                       dim=['fp_rr_advance', 'fp_birth_interval'],
                                       indices=[fp_rr_advance_indices, fp_rr_birth_interval_indices]), 
            'birthOrderAdv' : gb_shape("advance birth order",
                                       "2D float",
                                       dim=['fp_rr_advance', 'fp_rr_mothers_age'],
                                       indices=[fp_rr_advance_indices, fp_rr_mothers_age_indices]), 
            
        }, defaultDataTags=['fp_easydata']),

        
        FP_RRAgeBirthOrderResultTag : gb_shape("age and birth order result",
                                             "3D float", 
                                             dim=['fp_rr_birth_order', 'fp_rr_age_order', gb_year_dim],
                                             indices=[fp_rr_birth_order_indices, fp_rr_age_order_indices, gb_year_indices]), 
        FP_RRBirthIntervalResultTag : gb_shape("birth interval result",
                                             "2D float",
                                             dim=['fp_rr_birth_interval', gb_year_dim],
                                             indices=[fp_rr_birth_interval_indices, gb_year_indices]), 

    }


import psutil
import gc
from pympler import summary
from pympler import muppy
from pympler import refbrowser
import referrers
from fastapi import Request
from AvenirCommon.Logger import get_pt_log_level, set_pt_log_level, LogLevelCodes

     
global mem_base
mem_base = None

def setup_version_services(app):
    @app.get("/", tags=['Global'])
    def root():
        return f'Welcome to {app.title}. Version {app.version}' 


    @app.get("/Version", tags=['Global'])
    def Version():
        return app.version

    @app.post("/GetVersion", tags=['Global'])
    def GetVersion():
        return app.version
    
    @app.get("/SystemStats", tags=['Global'])
    def SystemStats():
        gc.collect()
        
        return {
            'cpuPercent':psutil.cpu_percent(), 
            'percentMemoryAvailable': 100*(psutil.virtual_memory().available / psutil.virtual_memory().total),
            'virtualMemory':psutil.virtual_memory()._asdict()
        }
    
    @app.get("/SetLogLevel/{level}", tags=["Global"])
    @app.post("/SetLogLevel/{level}", tags=["Global"])
    def SetLogLevel(level:LogLevelCodes, request:Request):
        return set_pt_log_level(level)
        
    @app.get("/GetLogLevel", tags=["Global"])
    @app.post("/GetLogLevel", tags=["Global"])
    def GetLogLevel(request:Request):
        return get_pt_log_level()
    
    @app.get("/ResetMemoryTrap", tags=['Global'])
    def ResetMemoryTrap():
        gc.collect()
        all_objects = muppy.get_objects()
        global mem_base
        mem_base = summary.summarize(all_objects)
        pass
    
    @app.get("/MemoryTrap", tags=['Global'])
    def MemoryTest():
        gc.collect()
        all_objects = muppy.get_objects()

        mem_trap = summary.summarize(all_objects)
        global old_summary
        diff_sum = summary.get_diff(mem_base, mem_trap) if mem_base else []
        diff_list = [rec for rec in diff_sum if (rec[1]+rec[2])>0]
        def output_function(o):
             return str(type(o))
        
        cb = refbrowser.ConsoleBrowser(mem_trap, maxdepth=2, str_func=output_function)
        # summary.print_(sum1)  
        new_diff_list = []
        o1_ids = {id(obj) for obj in mem_base}
        o2_ids = {id(obj): obj for obj in mem_trap}
        diff = [obj for obj_id, obj in o2_ids.items() if obj_id not in o1_ids]
        for obj in diff_list:
            ref = referrers.get_referrer_graph(
                    obj,
                    exclude_object_ids=[id(mem_base), id(mem_trap), id(diff_list), id(o2_ids)]
                )
            
            print(ref)
            new_diff_list.append(ref)
            
        
        return {
            'cpuPercent':psutil.cpu_percent(), 
            'percentMemoryAvailable': 100*(psutil.virtual_memory().available / psutil.virtual_memory().total),
            'virtualMemory':psutil.virtual_memory()._asdict(),
            'muppyLen': len(diff_list),
            'muppy':diff_list 
        }


import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.RP.RPModVarFields as rpmvf

#######################################################################################################
#                                                                                                     #
#                                       Sectors                                                       #
#                                                                                                     #
#######################################################################################################

def get_RP_sector_rec(sector_records = list, ID = str):
    sector_recs = [sector_rec for sector_rec in sector_records if sector_rec[rpmvf.RP_SECTOR_ID] == ID]
    return sector_recs[0] if len(sector_recs) > 0 else gbc.GB_NOT_FOUND
   
def set_RP_edu_sector_prim_sch_start_age(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_PRIM_SCH_START_AGE] = value

# Education

def set_RP_edu_sector_prim_sch_end_age(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_PRIM_SCH_END_AGE] = value

def set_RP_edu_sector_sec_sch_start_age(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_SEC_SCH_START_AGE] = value

def set_RP_edu_sector_sec_sch_end_age(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_SEC_SCH_END_AGE] = value

# def set_RP_edu_sector_student_per_teacher_prim(sector_rec = dict, value = float):
#     sector_rec[rpmvf.RP_SECTOR_EDU_STUDENT_PER_TEACHER_PRIM] = value

def get_RP_edu_sector_avg_ann_teacher_sal_prim_GDP_ratio(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_EDU_AVG_ANN_TEACHER_SAL_PRIM_GDP_RATIO]

def set_RP_edu_sector_avg_ann_teacher_sal_prim_GDP_ratio(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_AVG_ANN_TEACHER_SAL_PRIM_GDP_RATIO] = value

# def set_RP_edu_sector_avg_ann_teacher_sal_sec(sector_rec = dict, value = int):
#     sector_rec[rpmvf.RP_SECTOR_EDU_AVG_ANN_TEACHER_SAL_SEC] = value

def get_RP_edu_sector_prim_spend_per_student_perc_GDP(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_EDU_PRIM_SPEND_PER_STUDENT_PERC_GDP]

def set_RP_edu_sector_prim_spend_per_student_perc_GDP(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_PRIM_SPEND_PER_STUDENT_PERC_GDP] = value

def set_RP_edu_sector_sec_spend_per_student_perc_GDP(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_EDU_SEC_SPEND_PER_STUDENT_PERC_GDP] = value

# Health

def get_RP_health_sector_avg_unit_cost_skilled_deliv(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_UNIT_COST_SKILLED_DELIV]

def set_RP_health_sector_avg_unit_cost_skilled_deliv(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_UNIT_COST_SKILLED_DELIV] = value

def get_RP_health_sector_avg_unit_cost_ANC_4_plus_visits(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_UNIT_COST_ANC_4_PLUS_VISITS]

def set_RP_health_sector_avg_unit_cost_ANC_4_plus_visits(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_UNIT_COST_ANC_4_PLUS_VISITS] = value

def get_RP_health_sector_perc_skilled_hlth_prof_doctors(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_DOCTORS]

def set_RP_health_sector_perc_skilled_hlth_prof_doctors(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_DOCTORS] = value

def get_RP_health_sector_perc_skilled_hlth_prof_nurses(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_NURSES]

def set_RP_health_sector_perc_skilled_hlth_prof_nurses(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_NURSES] = value

def get_RP_health_sector_avg_salary_doctors(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_SALARY_DOCTORS]

def set_RP_health_sector_avg_salary_doctors(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_SALARY_DOCTORS] = value

def get_RP_health_sector_avg_salary_nurses(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_SALARY_NURSES]

def set_RP_health_sector_avg_salary_nurses(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_HLTH_AVG_SALARY_NURSES] = value

# WASH

def get_RP_WASH_sector_cap_costs_per_HH_impr_water(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_WATER]

def set_RP_WASH_sector_cap_costs_per_HH_impr_water(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_WATER] = value

def set_RP_WASH_sector_ann_cost_menstrual_hlth_comm(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_WASH_ANN_COST_MENSTRUAL_HLTH_COMM] = value

def get_RP_WASH_sector_cap_costs_per_HH_impr_sani(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_SANI]

def set_RP_WASH_sector_cap_costs_per_HH_impr_sani(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_SANI] = value

# Food security

def set_RP_food_sector_prim_staple_crop(sector_rec = dict, value = str):
    sector_rec[rpmvf.RP_SECTOR_FOOD_PRIM_STAPLE_CROP] = value

def set_RP_food_sector_agricult_land(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_FOOD_AGRICULT_LAND] = value

def get_RP_food_sector_crop_land(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_FOOD_CROP_LAND]

def set_RP_food_sector_crop_land(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_FOOD_CROP_LAND] = value

def get_RP_food_sector_staple_crop_prod_baseline(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_FOOD_STAPLE_CROP_PROD_BASELINE]

def set_RP_food_sector_staple_crop_prod_baseline(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_FOOD_STAPLE_CROP_PROD_BASELINE] = value

def set_RP_food_sector_area_harv_staple_crop(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_FOOD_AREA_HARV_STAPLE_CROP] = value

def get_RP_food_sector_tonnes_harv_perc_hect(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_FOOD_TONNES_HARV_PERC_HECT]

def set_RP_food_sector_tonnes_harv_perc_hect(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_FOOD_TONNES_HARV_PERC_HECT] = value

def get_RP_food_sector_ann_kg_per_year_consump_staple_crop(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_FOOD_ANN_KG_PER_YEAR_CONSUMP_STAPLE_CROP]

def set_RP_food_sector_ann_kg_per_year_consump_staple_crop(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_FOOD_ANN_KG_PER_YEAR_CONSUMP_STAPLE_CROP] = value

def get_RP_food_sector_prod_costs_staple_crop(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_FOOD_PROD_COSTS_STAPLE_CROP]

def set_RP_food_sector_prod_costs_staple_crop(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_FOOD_PROD_COSTS_STAPLE_CROP] = value

def get_RP_food_sector_daily_cost_food_aid(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_FOOD_DAILY_COST_FOOD_AID]

def set_RP_food_sector_daily_cost_food_aid(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_FOOD_DAILY_COST_FOOD_AID] = value

# Urbanization

def get_RP_urb_sector_perc_pop_urb_baseline_year(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_URB_PERC_POP_URB_BASELINE_YEAR]

def set_RP_urb_sector_perc_pop_urb_baseline_year(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_URB_PERC_POP_URB_BASELINE_YEAR] = value

def get_RP_urb_sector_perc_pop_urb_endline_year(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_URB_PERC_POP_URB_ENDLINE_YEAR]

def set_RP_urb_sector_perc_pop_urb_endline_year(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_URB_PERC_POP_URB_ENDLINE_YEAR] = value

def get_RP_urb_sector_cost_per_HH_move_to_housing(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_URB_COST_PER_HH_MOVE_TO_HOUSING]

def set_RP_urb_sector_cost_per_HH_move_to_housing(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_URB_COST_PER_HH_MOVE_TO_HOUSING] = value

# Energy

def set_RP_energy_sector_energy_KWh_prod_per_cap_per_year(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_ENERGY_KWH_PROD_PER_CAP_PER_YEAR] = value

def get_RP_energy_sector_cap_cost_per_HH_connect_grid(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_ENERGY_CAP_COSTS_PER_HH_CONNECT_GRID]

def set_RP_energy_sector_cap_cost_per_HH_connect_grid(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_ENERGY_CAP_COSTS_PER_HH_CONNECT_GRID] = value

def get_RP_energy_sector_cap_cost_per_HH_clean_cooking(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_ENERGY_CAP_COSTS_PER_HH_CLEAN_COOKING]

def set_RP_energy_sector_cap_cost_per_HH_clean_cooking(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_ENERGY_CAP_COSTS_PER_HH_CLEAN_COOKING] = value

def get_RP_energy_sector_cost_per_KWh(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_ENERGY_COST_PER_KWH]

def set_RP_energy_sector_cost_per_KWh(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_ENERGY_COST_PER_KWH] = value

# Economic

def get_RP_eco_sector_GDP_per_capita(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_ECO_GDP_PER_CAPITA]

def set_RP_eco_sector_GDP_per_capita(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_ECO_GDP_PER_CAPITA] = value

# def set_RP_eco_sector_baseline_GDP(sector_rec = dict, value = int):
#     sector_rec[rpmvf.RP_SECTOR_ECO_BASELINE_GDP] = value

# def set_RP_eco_sector_GDP_growth_rate(sector_rec = dict, value = float):
#     sector_rec[rpmvf.RP_SECTOR_ECO_GDP_GROWTH_RATE] = value

def get_RP_eco_sector_labor_partic_rate(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_ECO_LABOR_PARTIC_RATE]

def set_RP_eco_sector_labor_partic_rate(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_ECO_LABOR_PARTIC_RATE] = value

def get_RP_eco_sector_change_GDP_CDR(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_ECO_CHANGE_GDP_CDR]

def set_RP_eco_sector_change_GDP_CDR(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_ECO_CHANGE_GDP_CDR] = value

# Build Your Own

def get_RP_BYO_sector_targ_pop(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_BYO_TARG_POP]

def set_RP_BYO_sector_targ_pop(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_BYO_TARG_POP] = value

def get_RP_BYO_sector_multiplier_1(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_BYO_MULTIPLIER_1]

def set_RP_BYO_sector_multiplier_1(sector_rec = dict, value = int):
    sector_rec[rpmvf.RP_SECTOR_BYO_MULTIPLIER_1] = value

def get_RP_BYO_sector_unit_cost_2(sector_rec = dict):
    return sector_rec[rpmvf.RP_SECTOR_BYO_UNIT_COST_2]

def set_RP_BYO_sector_unit_cost_2(sector_rec = dict, value = float):
    sector_rec[rpmvf.RP_SECTOR_BYO_UNIT_COST_2] = value

#######################################################################################################
#                                                                                                     #
#                                       Goals                                                         #
#                                                                                                     #
#######################################################################################################

def get_RP_goal_rec(sector_rec = dict, ID = str):
    goal_recs = [goal_rec for goal_rec in sector_rec[rpmvf.RP_SECTOR_GOAL_RECORDS] if goal_rec[rpmvf.RP_GOAL_ID] == ID]
    return goal_recs[0] if len(goal_recs) > 0 else gbc.GB_NOT_FOUND

def get_RP_goal_selected_subgoal_ID(goal_rec = dict):
    return goal_rec[rpmvf.RP_GOAL_SELECTED_SUBGOAL_ID]

def get_RP_BYO_goal_resulting_change(goal_rec = dict, t = int):
    return goal_rec[rpmvf.RP_GOAL_RESULTING_CHANGE][t]

def set_RP_BYO_goal_resulting_change(goal_rec = dict, t = int, value = float):
    goal_rec[rpmvf.RP_GOAL_RESULTING_CHANGE][t] = value

#######################################################################################################
#                                                                                                     #
#                                       Subgoals                                                      #
#                                                                                                     #
#######################################################################################################

def get_RP_subgoal_rec(goal_rec = dict, ID = str):
    subgoal_recs = [subgoal_rec for subgoal_rec in goal_rec[rpmvf.RP_GOAL_SUBGOAL_RECORDS] if subgoal_rec[rpmvf.RP_SUBGOAL_ID] == ID]
    return subgoal_recs[0] if len(subgoal_recs) > 0 else gbc.GB_NOT_FOUND

def get_RP_subgoal_resulting_change(subgoal_rec = dict, t = int):
    return subgoal_rec[rpmvf.RP_SUBGOAL_RESULTING_CHANGE][t]

def set_RP_subgoal_resulting_change(subgoal_rec = dict, t = int, value = float):
    subgoal_rec[rpmvf.RP_SUBGOAL_RESULTING_CHANGE][t] = value

def get_RP_subgoal_resulting_change_by_sex(subgoal_rec = dict, sex = int, t = int):
    return subgoal_rec[rpmvf.RP_SUBGOAL_RESULTING_CHANGE][sex - 1][t]

def set_RP_subgoal_resulting_change_by_sex(subgoal_rec = dict, sex = int, t = int, value = float):
    subgoal_rec[rpmvf.RP_SUBGOAL_RESULTING_CHANGE][sex - 1][t] = value

#######################################################################################################
#                                                                                                     #
#                                       Climate parameters                                            #
#                                                                                                     #
#######################################################################################################

def get_RP_climate_baseline_mean_med_temp(climate_params_rec = dict):
    return climate_params_rec[rpmvf.RP_CLIM_BASELINE_MEAN_MED_TEMP]

def set_RP_climate_baseline_mean_med_temp(climate_params_rec = dict, value = float):
    climate_params_rec[rpmvf.RP_CLIM_BASELINE_MEAN_MED_TEMP] = value

def get_RP_climate_perc_change_GDP_1_degree_warm(climate_params_rec = dict):
    return climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_GDP_1_DEG_WARM]

def set_RP_climate_perc_change_GDP_1_degree_warm(climate_params_rec = dict, value = float):
    climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_GDP_1_DEG_WARM] = value

def get_RP_climate_perc_change_energy_needs_1_degree_warm(climate_params_rec = dict):
    return climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_ENERGY_NEEDS_1_DEG_WARM]

def set_RP_climate_perc_change_energy_needs_1_degree_warm(climate_params_rec = dict, value = float):
    climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_ENERGY_NEEDS_1_DEG_WARM] = value

def get_RP_climate_perc_change_staple_crop_1_degree_warm(climate_params_rec = dict):
    return climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_STAPLE_CROP_1_DEG_WARM]

def set_RP_climate_perc_change_staple_crop_1_degree_warm(climate_params_rec = dict, value = float):
    climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_STAPLE_CROP_1_DEG_WARM] = value

def get_RP_climate_perc_change_staple_crop_2_degree_warm(climate_params_rec = dict):
    return climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_STAPLE_CROP_2_DEG_WARM]

def set_RP_climate_perc_change_staple_crop_2_degree_warm(climate_params_rec = dict, value = float):
    climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_STAPLE_CROP_2_DEG_WARM] = value

def get_RP_climate_perc_change_staple_crop_3_degree_warm(climate_params_rec = dict):
    return climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_STAPLE_CROP_3_DEG_WARM]

def set_RP_climate_perc_change_staple_crop_3_degree_warm(climate_params_rec = dict, value = float):
    climate_params_rec[rpmvf.RP_CLIM_PERC_CHANGE_STAPLE_CROP_3_DEG_WARM] = value


from copy import deepcopy


import SpectrumCommon.Const.PJ.PJNTags as pjmvt

import SpectrumCommon.Const.IS.ISTags as ismvt
import SpectrumCommon.Const.IS.ISModVarFields as ismvf
import SpectrumCommon.Modvars.IS.ISReadDatabases as isrd

# Notes:
#
#   1. The only tangible purpose of versions on MVs is to be able to update them appropriately for existing projections so those projections can run with the
#      current codebase. The only code that ever checks MV versions is in the XXUpdateModvars file. If you just added a new field, you could just check and see if
#      the field is in the MV and add it if it's not without changing the version on the MV. Similarly, if you added a new dimension, you could just extend the
#      dimension of a MV without creating a new version. Changing the version number of a MV does gives us more of a history to follow, but doesn't do more
#      for us programmatically. Also, if you change the version of a MV, all clients using that MV need to update their code to use the latest MV.
#      Changing the version of a MV is only really useful if the MV undergoes a major structural change that would make it difficult to
#      manipulate the existing version of that MV otherwise, or if we would like to apply a correction to all previous versions of a MV.
#
#   2. You cannot base logic on whether a current MV tag exists in the code below.  It will always exist, because before this gets called, all missing MVs
#      will get added to the projection.
#
#   3. To trigger this, create a projection, say on the client, save, and grab it's projection ID (in Chrome, F12 --> Application --> session storage).
#      Change GB_PM_Version in GBConst or UpdateModvars will get skipped.  Add proj ID to Postman environment. Call Login, then GetModvars to get the library ID and put that
#      into the Postman environment. Then call OpenProjection. Make sure to leave the projection on the client open  while doing this because the proj ID changes when
#      you open/close. Then call SaveProjection. You should now be able to call Calculate and see the modified MV there where it's used. Don't forget to change
#      the version number back after you finish testing and trash the test projection.
#
def IS_UpdateModvars(projection, created_projection):

    if IS_FACILITY_TYPE_RECORDS_TAG_V1 in projection:
        _convert_facility_type_records_1(projection, IS_FACILITY_TYPE_RECORDS_TAG_V1)

    # This was a part of _convert_facility_type_records_1, but forgot that the IS module was being turned on with LiST projections (IS is needed for HW calcs),
    # so extend this code toall costing modes. Not changing the tag version for facility type records so the client/DPS doesn't need to change it too.
    if ismvt.IS_FACILITY_TYPE_RECORDS_TAG in projection:
        for rec in projection[ismvt.IS_FACILITY_TYPE_RECORDS_TAG]:
            # Before we only had a generic total equipment cost, but then we added maintenance and calibration costs. The sum of the three
            # is really the total equipment cost and total equipment cost is really the purchase cost of the equipment. Change the name so we
            # don't get confused.
            if 'totalEquipmentCost' in rec:
                rec['equipmentPurchaseCost'] = rec.pop('totalEquipmentCost')

    #####

    if IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V1 in projection:
        _convert_med_equip_group_records_1(projection, IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V1)
        _convert_med_equip_group_records_2(projection, ismvt.IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG)

    elif IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V2 in projection:
        _convert_med_equip_group_records_2(projection, IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V2)

    #####

    if IS_FACILITY_MED_EQUIP_RECORDS_TAG_V1 in projection:
        _convert_med_equip_records_1(projection, IS_FACILITY_MED_EQUIP_RECORDS_TAG_V1)
        _convert_med_equip_records_2(projection, ismvt.IS_FACILITY_MED_EQUIP_RECORDS_TAG)

    elif IS_FACILITY_MED_EQUIP_RECORDS_TAG_V2 in projection:
        _convert_med_equip_records_2(projection, IS_FACILITY_MED_EQUIP_RECORDS_TAG_V2)

    #####

    # delete tags
    for tag in IS_TagsForDeletion:
        if tag in projection:
            del projection[tag]

    return projection

IS_FACILITY_TYPE_RECORDS_TAG_V1                = '<IS_FacilityRecords_V1>'
IS_FACILITY_MED_EQUIP_RECORDS_TAG_V1           = '<IS_FacilityMedEquipRecords_V1>'
IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V1     = '<IS_FacilityMedEquipGroupRecords_V1>'
IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V2     = '<IS_FacilityMedEquipGroupRecords_V2>'
IS_FACILITY_MED_EQUIP_RECORDS_TAG_V2           = '<IS_FacilityMedEquipRecords_V2>'

IS_TagsForDeletion = [
    IS_FACILITY_TYPE_RECORDS_TAG_V1,
    IS_FACILITY_MED_EQUIP_RECORDS_TAG_V1,
    IS_FACILITY_MED_EQUIP_RECORDS_TAG_V2,
    IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V1,
    IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG_V2
]

def _convert_facility_type_records_1(projection, facility_type_records_tag):
    # New fields were added for TB app.
    facility_type_records = projection[facility_type_records_tag]

    if projection[pjmvt.GB_TBCostingOnTag]:
        for rec in facility_type_records:
            if ismvf.IS_FACIL_MED_EQUIP_MAINT_COST not in rec:
                rec[ismvf.IS_FACIL_MED_EQUIP_MAINT_COST] = 0

            if ismvf.IS_FACIL_MED_EQUIP_CALIB_COST not in rec:
                rec[ismvf.IS_FACIL_MED_EQUIP_CALIB_COST] = 0

            # Before we only had a generic total equipment cost, but then we added maintenance and calibration costs. The sum of the three
            # is really the total equipment cost and total equipment cost is really the purchase cost of the equipment. Change the name so we
            # don't get confused.
            if 'totalEquipmentCost' in rec:
                rec['equipmentPurchaseCost'] = rec.pop('totalEquipmentCost')

    if facility_type_records_tag != ismvt.IS_FACILITY_TYPE_RECORDS_TAG:
        projection[ismvt.IS_FACILITY_TYPE_RECORDS_TAG] = deepcopy(facility_type_records)

def _convert_med_equip_group_records_1(projection, med_equip_group_records_tag):
    # String constants were added for facility medical equipment group records.
    facility_MEG_records = projection[med_equip_group_records_tag]

    for rec in facility_MEG_records:
        if ismvf.IS_FACIL_MEG_STR_CONST not in rec:
            rec[ismvf.IS_FACIL_MEG_STR_CONST] = ''

    if med_equip_group_records_tag != ismvt.IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG:
        projection[ismvt.IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG] = deepcopy(facility_MEG_records)

def _convert_med_equip_group_records_2(projection, med_equip_group_records_tag):
    # String constants were added for facility medical equipment group records.
    facility_MEG_records = projection[med_equip_group_records_tag]
    facility_type_records = projection[ismvt.IS_FACILITY_TYPE_RECORDS_TAG]
    TB_costing_on = projection[pjmvt.GB_TBCostingOnTag]

    # Get the default medical equipment group records.
    facility_med_equip_group_DB = isrd.IS_load_facility_med_equip_group_DB()
    facility_MEG_records_DB = isrd.IS_init_facility_MEG_records(facility_med_equip_group_DB, TB_costing_on)

    # Make sure every facility type has at least one MEG record. If one is missing, add it.
    for fac_type_rec in facility_type_records:

        MEG_recs_for_fac_type = [MEG_rec for MEG_rec in facility_MEG_records if MEG_rec['facilityTypeID'] == fac_type_rec['facilityTypeID']]

        # If no MEG records were found for the facility type, add the default ones from the database.
        if len(MEG_recs_for_fac_type) == 0:
            MEG_recs_for_fac_type_DB = [MEG_rec for MEG_rec in facility_MEG_records_DB if MEG_rec['facilityTypeID'] == fac_type_rec['facilityTypeID']]

            for MEG_rec_DB in MEG_recs_for_fac_type_DB:
                facility_MEG_records.append(deepcopy(MEG_rec_DB))

    if med_equip_group_records_tag != ismvt.IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG:
        projection[ismvt.IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG] = deepcopy(facility_MEG_records)

def _convert_med_equip_records_1(projection, med_equip_records_tag):
    # String constants were added for facility equipment records.
    facility_equip_records = projection[med_equip_records_tag]

    for rec in facility_equip_records:
        if ismvf.IS_FACIL_EQUIP_STR_CONST not in rec:
            rec[ismvf.IS_FACIL_EQUIP_STR_CONST] = ''

    if med_equip_records_tag != ismvt.IS_FACILITY_MED_EQUIP_RECORDS_TAG:
        projection[ismvt.IS_FACILITY_MED_EQUIP_RECORDS_TAG] = deepcopy(facility_equip_records)

def _convert_med_equip_records_2(projection, med_equip_records_tag):
    # String constants were added for facility equipment records.
    facility_equip_records = projection[med_equip_records_tag]

    if projection[pjmvt.GB_TBCostingOnTag]:
        for rec in facility_equip_records:
            if ismvf.IS_FACIL_EQUIP_MAINT_COST not in rec:
                rec[ismvf.IS_FACIL_EQUIP_MAINT_COST] = 0

            if ismvf.IS_FACIL_EQUIP_CALIB_COST not in rec:
                rec[ismvf.IS_FACIL_EQUIP_CALIB_COST] = 0

    if med_equip_records_tag != ismvt.IS_FACILITY_MED_EQUIP_RECORDS_TAG:
        projection[ismvt.IS_FACILITY_MED_EQUIP_RECORDS_TAG] = deepcopy(facility_equip_records)



from SpectrumCommon.Const.IH.IHConst import *
from SpectrumCommon.Const.IH.IHTags import *
from SpectrumCommon.Const.IH.IHModVarFields import *

from SpectrumCommon.Const.IC.ICModVarFields import *
from SpectrumCommon.Const.IC.ICTags import *
from SpectrumCommon.Const.IC.ICInterventionIDs import *
from SpectrumCommon.Const.IC.IC_TI_InterventionIDs import *

from SpectrumCommon.Const.PC.PCModVarFields import *
from SpectrumCommon.Const.PC.PCTags import *

from SpectrumCommon.Const.IS.ISTags import *
from SpectrumCommon.Const.IS.ISModVarFields import *

from SpectrumCommon.Const.HW.HWTags import *
from SpectrumCommon.Const.HW.HWConst import *
from SpectrumCommon.Const.HW.HWModVarFields import *

#######################################################################################################
#                                                                                                     #
#                                      IH                                                             #
#                                                                                                     #
#######################################################################################################

BG_IH_DELIV_CHAN_RECORDS_TAG            = IH_DELIV_CHAN_RECORDS_TAG_V1
BG_IH_PROG_AREA_RECORDS_TAG             = IH_PROG_AREA_RECORDS_TAG_V1
BG_IH_COMMUNITY_CHAN_MST_ID             = IH_COMMUNITY_CHAN_MST_ID
BG_IH_SPECIFIC_PLAN                     = IH_SPECIFIC_PLAN
BG_IH_MODULE_MST_ID                     = IH_MODULE_MST_ID      
BG_IH_PROG_AREA_MST_ID                  = IH_PROG_AREA_MST_ID 
BG_IH_F_PROG_AREA_ID                    = IH_F_PROG_AREA_ID
BG_IH_HEALTH_SYS_MST_ID                 = IH_HEALTH_SYS_MST_ID
BG_IH_HEALTH_WORKFORCE_HSPM_MST_ID      = IH_HEALTH_WORKFORCE_HSPM_MST_ID
BG_IH_INFRASTRUCTURE_HSPM_MST_ID        = IH_INFRASTRUCTURE_HSPM_MST_ID
BG_IH_LOGISTICS_HSPM_MST_ID             = IH_LOGISTICS_HSPM_MST_ID
BG_IH_HEALTH_INFORM_SYSTEMS_HSPM_MST_ID = IH_HEALTH_INFORM_SYSTEMS_HSPM_MST_ID
BG_IH_PPLI_MST_ID                       = IH_PPLI_MST_ID
BG_IH_SOCIAL_ENABLERS_MST_ID            = IH_SOCIAL_ENABLERS_MST_ID
BG_IH_NCD_MST_ID                        = IH_NCD_MST_ID
BG_IH_HIV_AIDS_MST_ID                   = IH_HIV_AIDS_MST_ID

#######################################################################################################
#                                                                                                     #
#                                      IC                                                             #
#                                                                                                     #
#######################################################################################################

BG_IC_INTERV_RECORDS_TAG                       = IC_INTERV_RECORDS_TAG 
BG_IC_DRUG_SUPPLY_RECORDS_TAG                  = IC_DRUG_SUPPLY_RECORDS_TAG
BG_IC_RES_DRUGS_REQUIRED_TAG                   = IC_RES_DRUGS_REQUIRED_BY_DELIV_CHAN_TAG
BG_IC_RES_SUPPLIES_REQUIRED_TAG                = IC_RES_SUPPLIES_REQUIRED_BY_DELIV_CHAN_TAG
BG_IC_RES_DRUG_COSTS_TAG                       = IC_RES_DRUG_COSTS_TAG_V1
BG_IC_RES_SUPPLY_COSTS_TAG                     = IC_RES_SUPPLY_COSTS_TAG_V1
BG_IC_RES_LABOR_COSTS_DSC_TAG                  = IC_RES_LABOR_COSTS_DSC_TAG_V1
BG_IC_RES_OUTPAT_VISIT_INPAT_DAY_COSTS_DSC_TAG = IC_RES_OUTPAT_VISIT_INPAT_DAY_COSTS_DSC_TAG_V1
BG_IC_RES_LOG_SUPPLY_CHAIN_COSTS_TB_TAG        = IC_RES_LOG_SUPPLY_CHAIN_COSTS_TB_TAG
BG_IC_RES_WASTAGE_COSTS_TB_TAG                 = IC_RES_WASTAGE_COSTS_TB_TAG

BG_IC_F_DELIV_CHAN_ID              = IC_F_DELIV_CHAN_ID

#######################################################################################################
#                                                                                                     #
#                                      PC                                                             #
#                                                                                                     #
#######################################################################################################

BG_PC_COSTING_SECT_RECORDS_TAG       = PC_COSTING_SECT_RECORDS_TAG
BG_PC_ACT_RECORDS_TAG                = PC_ACT_RECORDS_TAG
BG_PC_COSTING_SECT_RECORDS_PPLI_TAG  = PC_COSTING_SECT_RECORDS_PPLI_TAG
BG_PC_ACT_RECORDS_PPLI_TAG           = PC_ACT_RECORDS_PPLI_TAG
BG_PC_RES_COSTS_BY_SECT_TAG          = PC_RES_COSTS_BY_SECT_TAG
BG_PC_RES_COSTS_BY_SECT_PPLI_TAG     = PC_RES_COSTS_BY_SECT_PPLI_TAG
BG_PC_RES_COSTS_BY_SUBSECT_TAG       = PC_RES_COSTS_BY_SUBSECT_TAG
BG_PC_RES_COSTS_BY_SUBSECT_PPLI_TAG  = PC_RES_COSTS_BY_SUBSECT_PPLI_TAG
BG_PC_RES_COSTS_BY_ACT_TAG           = PC_RES_COSTS_BY_ACT_TAG
BG_PC_RES_COSTS_BY_ACT_PPLI_TAG      = PC_RES_COSTS_BY_ACT_PPLI_TAG

BG_PC_F_COSTING_SECT_ID           = PC_F_COSTING_SECT_ID
BG_PC_F_COSTING_SUBSECT_ID        = PC_F_COSTING_SUBSECT_ID
BG_PC_F_ACT_ID                    = PC_F_ACT_ID

#######################################################################################################
#                                                                                                     #
#                                      IS                                                             #
#                                                                                                     #
#######################################################################################################

BG_IS_FACILITY_TYPE_RECORDS_TAG                       = IS_FACILITY_TYPE_RECORDS_TAG 
BG_IS_RES_TOTAL_COST_BUILD_EQUIP_NEW_FAC_TAG          = IS_RES_TOTAL_COST_BUILD_EQUIP_NEW_FAC_TAG                                                                                 
BG_IS_RES_VEHIC_PURCH_CAP_COSTS_TAG                   = IS_RES_VEHIC_PURCH_CAP_COSTS_TAG   
BG_IS_RES_VEHIC_PURCH_CAP_COSTS_BY_FAC_TAG            = IS_RES_VEHIC_PURCH_CAP_COSTS_BY_FAC_TAG
BG_IS_RES_VEHIC_PURCH_CAP_COSTS_OTHER_NON_FAC_TAG     = IS_RES_VEHIC_PURCH_CAP_COSTS_OTHER_NON_FAC_TAG        
BG_IS_RES_VEHIC_MAINT_REPAIR_COSTS_TAG                = IS_RES_VEHIC_MAINT_REPAIR_COSTS_TAG   
BG_IS_RES_VEHIC_MAINT_REPAIR_COSTS_BY_FAC_TAG         = IS_RES_VEHIC_MAINT_REPAIR_COSTS_BY_FAC_TAG
BG_IS_RES_VEHIC_MAINT_REPAIR_COSTS_OTHER_NON_FAC_TAG  = IS_RES_VEHIC_MAINT_REPAIR_COSTS_OTHER_NON_FAC_TAG                
BG_IS_RES_VEHIC_FUEL_COSTS_TAG                        = IS_RES_VEHIC_FUEL_COSTS_TAG  
BG_IS_RES_VEHIC_FUEL_COSTS_BY_FAC_TAG                 = IS_RES_VEHIC_FUEL_COSTS_BY_FAC_TAG
BG_IS_RES_VEHIC_FUEL_COSTS_OTHER_NON_FAC_TAG          = IS_RES_VEHIC_FUEL_COSTS_OTHER_NON_FAC_TAG                                
BG_IS_RES_VEHIC_DRIVER_SALARIES_TAG                   = IS_RES_VEHIC_DRIVER_SALARIES_TAG        
BG_IS_RES_VEHIC_DRIVER_SALARIES_BY_FAC_TAG            = IS_RES_VEHIC_DRIVER_SALARIES_BY_FAC_TAG
BG_IS_RES_VEHIC_DRIVER_SALARIES_OTHER_NON_FAC_TAG     = IS_RES_VEHIC_DRIVER_SALARIES_OTHER_NON_FAC_TAG              
BG_IS_RES_NEW_FAC_FURNITURE_COSTS_TAG                 = IS_RES_NEW_FAC_FURNITURE_COSTS_TAG            
BG_IS_RES_NEW_FAC_CONSTRUCT_COSTS_TAG                 = IS_RES_NEW_FAC_CONSTRUCT_COSTS_TAG       
BG_IS_RES_NEW_FAC_MED_EQUIP_COSTS_TAG                 = IS_RES_NEW_FAC_MED_EQUIP_COSTS_TAG       
BG_IS_RES_FAC_REHAB_COSTS_TAG                         = IS_RES_FAC_REHAB_COSTS_TAG                             
BG_IS_RES_FAC_TOTAL_OP_COSTS_TAG                      = IS_RES_FAC_TOTAL_OP_COSTS_TAG  
BG_IS_RES_ICT_CAP_COSTS_BY_FAC_TAG                    = IS_RES_ICT_CAP_COSTS_BY_FAC_TAG
BG_IS_RES_ICT_CAP_COSTS_OTHER_NON_FAC_TAG             = IS_RES_ICT_CAP_COSTS_OTHER_NON_FAC_TAG
BG_IS_RES_ICT_MAINT_REPAIR_OP_COSTS_BY_FAC_TAG        = IS_RES_ICT_MAINT_REPAIR_OP_COSTS_BY_FAC_TAG
BG_IS_RES_ICT_MAINT_REPAIR_OP_COSTS_OTHER_NON_FAC_TAG = IS_RES_ICT_MAINT_REPAIR_OP_COSTS_OTHER_NON_FAC_TAG
                                                                            
BG_IS_RES_EQUIPMENT_COSTS_DSC_V1   = IS_RES_EQUIPMENT_COSTS_DSC_V1

#######################################################################################################
#                                                                                                     #
#                                      HW                                                             #
#                                                                                                     #
#######################################################################################################

BG_HW_STAFF_TYPE_RECORDS_TAG                   = HW_STAFF_TYPE_RECORDS_TAG_V1
BG_HW_INSERV_TRAIN_TYPE_RECORDS_TAG            = HW_INSERV_TRAIN_TYPE_RECORDS_TAG_V1
BG_HW_RES_TOTAL_COMPENSATION_TAG               = HW_RES_TOTAL_COMPENSATION_TAG_V1
BG_HW_RES_PRESERV_TRAIN_COSTS_TAG              = HW_RES_PRESERV_TRAIN_COSTS_TAG_V1
BG_HW_RES_INSERV_TRAIN_COSTS_BY_STAFF_TAG      = HW_RES_INSERV_TRAIN_COSTS_BY_STAFF_TAG_V1
BG_HW_RES_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG = HW_RES_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG_V1

BG_HW_NUM_STAFFING_LEVELS  = HW_NUM_STAFFING_LEVELS
BG_HW_STAFF_LEVELS_MST_IDS = HW_STAFF_LEVELS_MST_IDS
BG_HW_STAFF_LEVEL_MAP      = HW_STAFF_LEVEL_MAP

BG_HW_NUM_COMPENS_TYPES    = HW_NUM_COMPENS_TYPES
BG_HW_COMPENS_TYPE_MST_IDS = HW_COMPENS_TYPE_MST_IDS
BG_HW_COMPENS_TYPE_MAP     = HW_COMPENS_TYPE_MAP

BG_HW_NUM_INSERV_TRAIN_TYPES = HW_NUM_INSERV_TRAIN_TYPES



from os import environ

from loguru import logger
import numpy as np

from AvenirCommon.Database import GB_get_db_json
from AvenirCommon.Util import GBGetKeyValue

from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.PJ import *
from SpectrumCommon.Modvars.GB.GBUtil import  getYearIdx
from SpectrumCommon.Const.GB import GB_UseInflation, GB_NumInflationTypes, GB_Female, GB_DW_MaxAgeGrp, GB_JSON

import SpectrumCommon.Const.IH as ihc
import SpectrumCommon.Const.IH.IHDatabaseKeys as ihdbk


# Please keep for now in case they change their minds

# Initializes exchange rate using IH values instead of GB values if IHT is on.
def init_exchange_infl_rate_IHT(countryID):
    environment = environ[GB_SPECT_MOD_DATA_CONN_ENV]
    exchange_rate_DB_file_name = ihc.IH_EXCHANGE_RATE_DB_DIR.replace('\\', '/') + '/' + countryID + '_' + ihc.IH_EXCHANGE_RATE_DB_CURR_VERSION + '.' + GB_JSON
    exchange_rate_DB = GB_get_db_json(environment, GB_IH_CORE_CONTAINER, exchange_rate_DB_file_name)

    return exchange_rate_DB[ihdbk.IH_EXCHANGE_RATE_KEY_EXDB], exchange_rate_DB[ihdbk.IH_INFLATION_RATE_KEY_EXDB]

def init_modvars(countryID, first_year, final_year, extra):
    modules = GBGetKeyValue(extra, 'modules', '')

    # Please keep for now in case they change their minds
    if (GB_IH in modules) or (GB_IC in modules):
        exchange_rate_val, domestic_inflation_rate_val = init_exchange_infl_rate_IHT(countryID)
    else:
        exchange_rate_val = extra['countryDetails']['exchangeRate']
        domestic_inflation_rate_val = 0.0

    exchange_rate = np.full(getYearIdx(final_year, first_year) + 1, exchange_rate_val).tolist()
    inflation = np.zeros((getYearIdx(final_year, first_year) + 1, GB_NumInflationTypes))
    inflation[:, GB_InflationLocal - 1] = domestic_inflation_rate_val
    inflation[:, GB_InflationUS - 1] = 3.0
    inflation = inflation.tolist()

    LiST_costing_on = extra.get(GB_LIST_COSTING_ON, False)
    GSKMode         = extra.get(GB_GSKMode, False)
    TB_costing_on   = extra.get(GB_TB_COSTING_ON, False)
    TB_statistical  = extra.get(GB_TB_ImpactStatistical, True)
    TB_dynamical    = extra.get(GB_TB_ImpactDynamical, False)
    useLeapfrog     = extra.get(GB_Extra_UseLeapfrog, False)
    # IHT costing is on if it was passed in and true or if both LiST and TB costing on are not on. This makes it so we
    # can be lazy and not have to specify IHT costing on for an IHT projection.
    if ((GB_IHT_ON in extra) and (extra[GB_IHT_ON])) or (not (LiST_costing_on or TB_costing_on)):
        IHT_costing_on = True
    else:
        IHT_costing_on = False
    # RAPID_sectors_on = extra.get(GB_RAPID_SECTORS_ON, RP_init_sector_list())

    #modvar creation list
    modvars = {
            # db,
        GB_UseLeapfrogTag: useLeapfrog,

        GB_DataChangedTag : False,
        GB_CalcStateValidTag : False,

        #country info
        GB_CustomEasyDataSetTag : [],

        GB_CurrencyNameTag : extra['countryDetails']['currency'], # Not sure what "currencyName" is, but currency value matches what we used to see in IHT, so switching to that
        GB_DiscountRateCostsTag : 3.0,
        GB_ExchangeRateTag : exchange_rate,
        GB_InflationTag : inflation,
        GB_InflationMethodTag : GB_UseInflation,

        GB_DisabilityWeightsTag : np.zeros((GB_Female + 1, GB_DW_MaxAgeGrp + 1)).tolist(),
        GB_DisabilityWeightsSetTag : [],

        GB_LCOnTag : LiST_costing_on,
        GB_GSKModeTag : GSKMode,
        GB_TBCostingOnTag : TB_costing_on,
        GB_IHTOnTag : IHT_costing_on,
        # GB_RAPIDSectorsOnTag : RAPID_sectors_on,

        GB_SubnatWizProjTag : 'NotImplemented',
        GB_SubnatRegionTag  : 'NotImplemented',
        GB_SubnatSurveydTag : 'NotImplemented',

        GB_TBImpactStatisticalTag : TB_statistical,
        GB_TBImpactDynamicalTag   : TB_dynamical,
        GB_UserSourcesTag : '',
        GB_TaskGUIDsTag : [],
        # GB_CloseStateDBProjTag : True #EPP will turn this False
    }

    return modvars









from copy import deepcopy


import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.PC.PCTags as pcmvt
import SpectrumCommon.Const.PC.PCModVarFields as pcmvf
import SpectrumCommon.Modvars.PC.PCReadDatabases as pcrd
import SpectrumCommon.Const.PC.PCDatabaseKeys as pcdbk
import SpectrumCommon.Modvars.PC.PCModVarUtil as pcmvu

# Notes:
#
#   1. You cannot base logic on whether a current MV tag exists in the code below.  It will always exist, because before this gets called, all missing MVs
#      will get added to the projection.
#
#   2. To trigger this, create a projection, say on the client, save, and grab it's projection ID.  Call GetModvars to get the library ID and put that into the appropriate
#      environment variable. Then call OpenProjection. Make sure to leave the projection on the client open while doing this because the proj ID changes when you open/close.
def PC_UpdateModvars(projection, created_projection):

    if PC_COST_INPUT_RECORDS_TAG_V1 in projection:
        _convert_cost_input_records_1(projection, PC_COST_INPUT_RECORDS_TAG_V1)
        _convert_cost_input_records_2(projection, pcmvt.PC_COST_INPUT_RECORDS_TAG)

    elif PC_COST_INPUT_RECORDS_TAG_V2 in projection:
        _convert_cost_input_records_2(projection, PC_COST_INPUT_RECORDS_TAG_V2)

    if PC_COSTING_SECT_RECORDS_TAG_V1 in projection:
        _convert_costing_sect_records_1(projection, PC_COSTING_SECT_RECORDS_TAG_V1)

    if PC_ACT_RECORDS_TAG_V1 in projection:
        _convert_act_records_1(projection, PC_ACT_RECORDS_TAG_V1)

    # A bug was discovered which caused activity records and line item records not to be removed when corresponding costing sections and subsections were deleted.
    # This function corrects this error by removing activities and line items which do not have a matching costing section or subsection.
    _remove_deleted_activities_and_line_items(projection)

    # delete tags
    for tag in PC_TagsForDeletion:
        if tag in projection:
            del projection[tag]

    return projection

PC_COST_INPUT_RECORDS_TAG_V1    = '<PC_CostInputRecords_V1>'
PC_COST_INPUT_RECORDS_TAG_V2    = '<PC_CostInputRecords_V2>'
PC_COSTING_SECT_RECORDS_TAG_V1  = '<PC_CostingSectionRecords_V1>'
PC_ACT_RECORDS_TAG_V1           = '<PC_ActivityRecords_V1>'

PC_TagsForDeletion = [
    PC_COST_INPUT_RECORDS_TAG_V1,
    PC_COST_INPUT_RECORDS_TAG_V2,
    PC_COSTING_SECT_RECORDS_TAG_V1,
    PC_ACT_RECORDS_TAG_V1,
]

def _reload_string_constants_cost_inputs(cost_input_records_value):
    cost_input_DB = pcrd.PC_load_cost_input_DB()

    # Create a dict out of the database for fast lookups. Key-value pairs are input ID - inner dicts where the inner dicts are
    # cost input string constant IDs and unit of measure string constant IDs
    cost_input_DB_dict = {}

    for cost_input_DB_obj in cost_input_DB:
        cost_input_DB_dict[str(cost_input_DB_obj[pcdbk.PC_COST_INPUT_ID_KEY_CIDB])] = {
            'costInputStringConstant'     : cost_input_DB_obj[pcdbk.PC_COST_INPUT_STR_CONST_KEY_CIDB],
            'unitOfMeasureStringConstant' : cost_input_DB_obj[pcdbk.PC_UNIT_OF_MEASURE_STR_CONST_KEY_CIDB],
        }

    for rec in cost_input_records_value:
        cost_input_ID = rec[pcmvf.PC_COST_INPUT_ID]

        if cost_input_ID in cost_input_DB_dict:
            rec[pcmvf.PC_COST_INPUT_STR_CONST] = cost_input_DB_dict[cost_input_ID]['costInputStringConstant']
            rec[pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST] = cost_input_DB_dict[cost_input_ID]['unitOfMeasureStringConstant']

def _reload_string_constants_cost_sections_and_subsections(cost_section_records_value):
    # Both come from the same database.
    cost_section_DB = pcrd.PC_load_costing_sect_DB()

    ##########   Costing sections   ###########

    # Create a dict out of the database for fast lookups. Key-value pairs are keys described below and costing string constants.
    cost_sections_DB_dict = {}

    for cost_section_DB_obj in cost_section_DB:
        # Costing section rows will have a subsection ID of -1
        costing_subsect_ID = cost_section_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB]

        if costing_subsect_ID == -1:
            # Costing sections require area type ID, area ID, and costing section ID to positively identify them.
            key = (str(
                str(cost_section_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_CSDB])
                + '_' + str(cost_section_DB_obj[pcdbk.PC_AREA_ID_KEY_CSDB])
                + '_' + str(cost_section_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_CSDB])
            ))
            cost_sections_DB_dict[key] = cost_section_DB_obj[pcdbk.PC_COSTING_SECT_STR_CONST_KEY_CSDB]

    for rec in cost_section_records_value:
        area_type_ID = rec[pcmvf.PC_COSTING_SECT_AREA_TYPE_ID]
        area_ID = rec[pcmvf.PC_COSTING_SECT_AREA_ID]
        costing_sect_ID = rec[pcmvf.PC_COSTING_SECT_ID]

        key = str(str(area_type_ID) + '_' + str(area_ID) + '_' + str(costing_sect_ID))

        if key in cost_sections_DB_dict:
            rec[pcmvf.PC_COSTING_SECT_STR_CONST] = cost_sections_DB_dict[key]

    ##########   Costing subsections   ###########

    cost_subsection_DB_dict = {}

    for cost_subsection_DB_obj in cost_section_DB:
        # Costing subsection rows will have a subsection ID other than -1
        costing_subsect_ID = cost_subsection_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB]

        if costing_subsect_ID != -1:
            # Costing sections require area type ID, area ID, and costing section ID to positively identify them. Costing subsections require these in addition to
            # the costing subsection ID.
            key = (str(
                str(cost_subsection_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_CSDB])
                + '_' + str(cost_subsection_DB_obj[pcdbk.PC_AREA_ID_KEY_CSDB])
                + '_' + str(cost_subsection_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_CSDB])
                + '_' + str(cost_subsection_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB])
            ))
            cost_subsection_DB_dict[key] = cost_subsection_DB_obj[pcdbk.PC_COSTING_SUBSECT_STR_CONST_KEY_CSDB]

    for section_rec in cost_section_records_value:
        area_type_ID = section_rec[pcmvf.PC_COSTING_SECT_AREA_TYPE_ID]
        area_ID = section_rec[pcmvf.PC_COSTING_SECT_AREA_ID]
        costing_sect_ID = section_rec[pcmvf.PC_COSTING_SECT_ID]
        subsect_records = section_rec[pcmvf.PC_COSTING_SECT_SUBSECT_RECORDS]

        for subsect_rec in subsect_records:
            subsect_ID = subsect_rec[pcmvf.PC_COSTING_SUBSECT_ID]

            key = str(str(area_type_ID) + '_' + str(area_ID) + '_' + str(costing_sect_ID) + '_' + str(subsect_ID))

            if key in cost_subsection_DB_dict:
                subsect_rec[pcmvf.PC_COSTING_SUBSECT_STR_CONST] = cost_subsection_DB_dict[key]

def _convert_cost_input_records_1(projection, cost_input_records_tag):
    cost_input_records_value = projection[cost_input_records_tag]

    # String constants were added.

    for rec in cost_input_records_value:
        if pcmvf.PC_COST_INPUT_STR_CONST not in rec:
            rec[pcmvf.PC_COST_INPUT_STR_CONST] = ''

        if pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST not in rec:
            rec[pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST] = ''

    if cost_input_records_tag != pcmvt.PC_COST_INPUT_RECORDS_TAG:
        projection[pcmvt.PC_COST_INPUT_RECORDS_TAG] = deepcopy(cost_input_records_value)

def _convert_cost_input_records_2(projection, cost_input_records_tag):
    cost_input_records_value = projection[cost_input_records_tag]

    _reload_string_constants_cost_inputs(cost_input_records_value)

    if cost_input_records_tag != pcmvt.PC_COST_INPUT_RECORDS_TAG:
        projection[pcmvt.PC_COST_INPUT_RECORDS_TAG] = deepcopy(cost_input_records_value)

def _convert_act_records_1(projection, act_records_tag):
    act_records_value = projection[act_records_tag]

    for rec in act_records_value:
        if pcmvf.PC_ACT_STR_CONST not in rec:
            rec[pcmvf.PC_ACT_STR_CONST] = ''

    if act_records_tag != pcmvt.PC_ACT_RECORDS_TAG:
        projection[pcmvt.PC_ACT_RECORDS_TAG] = deepcopy(act_records_value)

def _convert_costing_sect_records_1(projection, costing_sect_records_tag):
    cost_section_records_value = projection[costing_sect_records_tag]

    _reload_string_constants_cost_sections_and_subsections(cost_section_records_value)

    if costing_sect_records_tag != pcmvt.PC_COSTING_SECT_RECORDS_TAG:
        projection[pcmvt.PC_COSTING_SECT_RECORDS_TAG] = deepcopy(cost_section_records_value)

def _remove_deleted_activities_and_line_items(projection):
    def remove_deleted_line_items(line_item_mv_tag, section_mv_tag):
        # Removes line items (cost inputs) that do not have a matching section or subsection
        if line_item_mv_tag in projection:
            ALI_records = projection[line_item_mv_tag]
            existing_section_records = projection[section_mv_tag]

            for ALI_rec in ALI_records:
                costing_sect_ID = ALI_rec[pcmvf.PC_ALI_COSTING_SECT_ID]
                costing_subsect_ID = ALI_rec[pcmvf.PC_ALI_COSTING_SUBSECT_ID]
                area_ID = ALI_rec[pcmvf.PC_ALI_AREA_ID]
                area_type_ID = ALI_rec[pcmvf.PC_ALI_AREA_TYPE_ID]
                act_ID = ALI_rec[pcmvf.PC_ALI_ACT_ID]
                cost_input_ID = ALI_rec[pcmvf.PC_ALI_COST_INPUT_ID]

                matching_section = pcmvu.get_PC_costing_sect_rec(existing_section_records, area_type_ID, area_ID, costing_sect_ID)
                matching_subsection = pcmvu.get_PC_costing_subsect_rec(existing_section_records, area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID)

                if matching_section == gbc.GB_NOT_FOUND or matching_subsection == gbc.GB_NOT_FOUND:
                    ALI_records = pcmvu.remove_PC_ALI_rec(ALI_records, area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID, act_ID, cost_input_ID)
            
            projection[line_item_mv_tag] = deepcopy(ALI_records)

    def remove_deleted_activities(activity_mv_tag, section_mv_tag):
        # Removes activities that do not have a matching section or subsection
        if activity_mv_tag in projection:
            act_records = projection[activity_mv_tag]
            existing_section_records = projection[section_mv_tag]

            for activity_rec in act_records:
                costing_sect_ID = activity_rec[pcmvf.PC_ACT_COSTING_SECT_ID]
                costing_subsect_ID = activity_rec[pcmvf.PC_ACT_COSTING_SUBSECT_ID]
                area_ID = activity_rec[pcmvf.PC_ACT_AREA_ID]
                area_type_ID = activity_rec[pcmvf.PC_ACT_AREA_TYPE_ID]
                act_ID = activity_rec[pcmvf.PC_ACT_ID]

                matching_section = pcmvu.get_PC_costing_sect_rec(existing_section_records, area_type_ID, area_ID, costing_sect_ID)
                matching_subsection = pcmvu.get_PC_costing_subsect_rec(existing_section_records, area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID)

                if matching_section == gbc.GB_NOT_FOUND or matching_subsection == gbc.GB_NOT_FOUND:
                    act_records = pcmvu.remove_PC_act_rec(act_records, area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID, act_ID)

            projection[activity_mv_tag] = deepcopy(act_records)

    # Needs to be done for both regular programme costing and PPLI, as they have separate modvars
    remove_deleted_line_items(pcmvt.PC_ACT_LINE_ITEM_RECORDS_TAG, pcmvt.PC_COSTING_SECT_RECORDS_TAG)
    remove_deleted_line_items(pcmvt.PC_ACT_LINE_ITEM_RECORDS_PPLI_TAG, pcmvt.PC_COSTING_SECT_RECORDS_PPLI_TAG)

    remove_deleted_activities(pcmvt.PC_ACT_RECORDS_TAG, pcmvt.PC_COSTING_SECT_RECORDS_TAG)
    remove_deleted_activities(pcmvt.PC_ACT_RECORDS_PPLI_TAG, pcmvt.PC_COSTING_SECT_RECORDS_PPLI_TAG)


FS_ScenarioNameTag = '<FS_ScenarioNames_V1>'
FS_MacroGDPTag = '<FS_MacroGDP_V1>'
FS_MacroExpenditureTag = '<FS_MacroExpenditure_V1>'
FS_MacroTaxesTag = '<FS_MacroTaxes_V1>'
FS_MacroNonTaxesTag = '<FS_MacroNonTaxes_V1>'
FS_MacroGrantsTag = '<FS_MacroGrants_V1>'
FS_MacroOtherRevenueTag = '<FS_MacroOtherRevenue_V1>'
FS_GGHEApproachTag = '<FS_GGHEApproach_V1>'
FS_GGHEDomesticTag = '<FS_GGHEDomestic_V1>'
FS_GGHEExternalTag = '<FS_GGHEExternal_V1>'
FS_HealthExpCoeffTag = '<FS_HealthExpCoeff_V1>'
FS_PercOfGGETag = '<FS_PercOfGGE_V1>'
FS_ConstantPricesTag = '<FS_ConstantPrices_V1>'
FS_OutOfPocketTag = '<FS_OutOfPocket_V1>'
FS_PrePaidRiskPlansTag = '<FS_PrePaidRiskPlans_V1>'
FS_GrowthOptionTag = '<FS_GrowthOption_V1>'
FS_PrivateHealthCoeffTag = '<FS_PrivateHealthCoeff_V1>'
FS_HealthExpOtherTag = '<FS_HealthExpOther_V1>'